;---------------------------------------------
;FUNCTION - CLEAR SCREEN PIXELS, SETS ALL THE PIXELS OF THE SCREEN BLACK (WONT CLEAR THE SPRITES)
;---------------------------------------------
CLEANSCREENPIXELS:
	;CLEAR THE SCREEN 
	;DRAW BLACK PIXELS ON THE WHOLE 320X240 PIXELS
	SetVeraAddr $100000 ;Bitmap ram $-0 00 00 (Autoinc $01)
	
	ldx #$01+1			;320x240=$12C00
	ldy #$2C+1
	lda #$00+1
TestPatternLoop:	
	stz V_D1			;WRITE $00 TO SCREEN MEMORY
	sec
	sbc #1
	bne TestPatternLoop
	dey
	bne TestPatternLoop
	dex
	bne TestPatternLoop
	
	RTS						;RETURN TO MAIN GAME LOOP

;---------------------------------------------
;FUNCTION - HIDEPIECE, THIS HIDES/REMOVES A SPRITE OF THE SCREEN, SO IT BECOMES UNSEEN, A CONTAINS SPRITE NUMBER
;---------------------------------------------
HIDEPIECE:
	;A ALREADY CONTAINS THE SPRITE NUMBER
	LDX #15					;LOAD 15 TO THE X REGISTER
	LDY #15					;LOAD 15 TO THE Y REGISTER
	JSR MOVEPIECE			;MOVE THE PIECE TO THE NEW LOCATION (VARIABLES ONLY)
	LDX #3					;LOAD 3 INTO X REGISTER
	STX VAR_SPRITESTATUS	;STORE 3 TO THE VAR_SPRITESTATUS VARIABLE, THIS WILL TELL THE SHOWSPRITE-SETSPRITE SUB TO DISABLE THE SPRITE (FOR NOW)
	JSR SHOWPIECE			;ACTUALLY MOVE THE SPRITE TO THE NEW, OFFSCREEN LOCATION
	RTS						;ALL DONE, RETURN TO PREVIOUS CODE

;---------------------------------------------
;FUNCTION - MOVEPIECE, MOVES A PIECE ON THE BOARD, A CONTAINS THE SPRITE NUMBER, X CONTAINS THE X COORDINATE, Y CONTAINS Y COORDINATE
;---------------------------------------------
MOVEPIECE
	PHA					;STORE A IN THE STACK (BACKUP)
	
		JSR GETSPRITEDATA	;LOAD THE CURRENT SPRITE DATA; (THATS THEN ALL STORED IN ZPADDR - ZPADDR+7)

		TXA					;TRANSFER THE X POSITION TO A. A NOW CONTAINS 0000XXXX
		ASL					;SHIFT THE BITS TO THE LEFT, ONCE,
		ASL					;..TWICE..
		ASL					;..THREE TIMES..
		ASL					;AND ONE MORE TIME. A NOW CONTAINS XXXX0000
		STA	ZPADDR			;TEMPORARY STORE A IN ZPADDR
		TYA					;TRANSFER Y POSITION TO A (0000YYYY)
		CLC
		ADC ZPADDR			;ADD THE VALUE OF ZPADDR (XXXX0000) TO A, SO THE RESULT IS NOW XXXXYYYY
		STA ZPADDR			;STORE IT IN ZPADDR FOR NOW.

		LDA ZPADDR+5		;LOAD THE LOW BYTE OF THE START ADDRESS OF THE SPRITE DATA ARRAY RAM (1ST BYTE)
		CLC
		ADC #1				;ADD 1 (BECAUSE WE NEED TO POINT TO THE 2ND BYTE), THE LOW BYTE OF THE MEMORY ADDRESS TO STORE THE X AND Y POSITION IN.
		STA ZPADDR+1		;STORE THE LOW BYTE IN ZPADDR+1
		LDA #SPRITEARRAYH	;LOAD $40 IN A (HIGH BYTE OF SPRITE DATA ARRAY)
		STA ZPADDR+2		;STORE IN ZPADDR+2, THE HIGH BYTE OF THE RAM MEMORY ADDRESS TO STORE THE X AND Y POSITION IN.
		LDA ZPADDR			;RELOAD ZPADDR (XXXXYYYY) INTO A
		STA (ZPADDR+1)		;STORE THE X AND Y POSITION IN RAM ADDRESS THAT ZPADDR+1 AND ZPADDR+2 POINT TO.
		
		LDA #0
		STA VAR_SPRITESTATUS	;STORE 0 IN VAR_SPRITESTATUS. WE'LL USE THAT TO TELL THE SETSPRITE FUNCTION TO LOAD A 16X16 SPRITE
	
	PLA					;RETRIEVE A FROM THE ACCUMULATOR AGAIN
MOVEPIECEY
	RTS					;RETURN TO PREVIOUS CODE

;---------------------------------------------
;FUNCTION - SET PIECE TYPE, SETS THE PIECE TYPE, A CONTAINS SPRITE NUMBER, X CONTAINS THE PIECE TYPE
;(0=BOMB,1=FLAG,2=SPY,3=SCOUT,4=MINER,5=SERGEANT,6=LIEUTENANT,7=CAPTAIN,8=MAJOR,9=COLONEL,10=GENERAL,11=MARSHALL) - TYPE 13 IS USED FOR THE GAME CURSOR
;STORED IN LOW NIBBLE OF 3RD BYTE OF SPRITE DATA ARRAY. ($0400 + (SPRITE NUMBER * 3) + 2 (FOR 3RD BYTE))
;---------------------------------------------
;NORMALLY SKIP OVER THIS FUNCTION
SETPIECETYPE
	PHA						;STORE A IN THE STACK (BACKUP)
	
		STA ZPADDR 			;STORE A IN ZPADDR FOR NOW (SPRITE NUMBER)
		ASL					;BIT SHIFT A TO THE LEFT (MULTIPLY SPRITE NUMBER BY 2)
		CLC					;CLEAR CARRY SO WE CAN ADD
		ADC ZPADDR			;ADD THE SPRITE NUMBER TO A (NOW THE SPRITE NUMBER IS * 3)
		CLC					;CLEAR CARRY FOR ADDING
		ADC #2				;ADD DECIMAL 2, SO THE SPRITE NUMBER WILL NOW POINT TO THE 3RD BYTE OF THE SPRITE ARRAY
		
		STA ZPADDR			;STORE THE RESULT IN LOW BYTE OF ZPADDR
		LDA #SPRITEARRAYH	;LOAD $40 IN A (HIGH BYTE OF SPRITE DATA ARRAY)
		STA ZPADDR+1		;STORE $04 IN THE HIGH BYTE OF ZPADDR, SO ZPADDR WILL NOW CONTAIN (IN LITTLE ENDIAN) $04 AND ((SPRITE NUMBER * 3) + 2)
		TXA					;TRANSFER THE PIECE TYPE TO A
		STA (ZPADDR)		;STORE THE SPRITE TYPE IN THE MEMORY LOCATION OF THE SPRITE
		
	PLA						;RETRIEVE A FROM THE ACCUMULATOR AGAIN
	RTS						;RETURN TO PREVIOUS CODE

;---------------------------------------------
;FUNCTION - SHOW PIECE ON SCREEN. THIS FEATURE WILL CALL THE SETSPRITE FUNCTION, A CONTAINS SPRITE NUMBER.
;ALL SPRITE INFORMATION (VERA ADDRESS, X POSITION, Y POSITION,) WILL BE TAKEN FROM THE SPRITE DATA ARRAY IN RAM.
;---------------------------------------------
;NORMALLY SKIP OVER THIS FUNCTION
SHOWPIECE
	PHA						;STORE A IN THE STACK (BACKUP)
	
		JSR GETSPRITEDATA	;LOAD THE CURRENT SPRITE DATA;
		
		;FIRST CHECK IF THE CURSOR HAS TO BE LOADED
		LDA ZPADDR
		CMP #80				;CHECK IF THE SPRITE NUMBER IS THE CURSOR
		BEQ SHOWPIECE_1_NORMAL	;IF IT IS, SHOW THE SPRITE AS NORMAL
		CMP #81				;CHECK IF THE SPRITE NUMBER IS THE PREVIOUSLY SELECTED PLAYER CURSOR
		BEQ SHOWPIECE_1_NORMAL	;IF IT IS, SHOW THE SPRITE AS NORMAL
		
		;SPECIAL CASE, IF VAR_ATTACK IS 1 THEN SHOW THE NORMAL FRONT SIDE OF THE COMPUTER'S PIECES. WE DONT NEED TO TEST FOR COMPUTER/PLAYER PIECE, JUST SHOW IT NORMALLY
		LDA VAR_ATTACK
		CMP #1
		BEQ SHOWPIECE_1_NORMAL
		;WHEN THE COMPUTER PIECES ARE PLACED, THEN THE BACKSIDE OF THE PIECE HAS TO BE SHOWN
		LDA ZPADDR 			;LOAD THE SPRITE NUMBER
		CMP #40				;CHECK IF THE SPRITE NUMBER IS EQUAL TO 40
		BCS SHOWPIECE_1_BLUE	;IF THE SPRITE NUMBER IS THE SAME OR ABOVE 40 (BLUE PIECE) THEN GOTO SHOWPIECE_1_BLUE
		LDA VAR_TEAM		;CHECK WHICH TEAM THE PLAYER PLAYES AT.
		CMP #0				;IF THE PLAYER IS TEAM 1 (RED)
		BEQ SHOWPIECE_1_NORMAL	;IF SO, THEN PRINT THE SPRITE AS NORMAL, SHOW THE FRONT OF THE PIECE
		BNE SHOWPIECE_1_R_BACK	;IF NOT, THEN PRINT THE BACK OF THE RED PIECES
		
SHOWPIECE_1_BLUE
		LDA VAR_TEAM		;CHECK WHICH TEAM THE PLAYER PLAYES AT.
		CMP #1				;IF THE PLAYER IS TEAM 1 (BLUE)
		BEQ SHOWPIECE_1_NORMAL	;IF SO, THEN PRINT THE SPRITE AS NORMAL, SHOW THE FRONT OF THE PIECE
		;IF NOT, THEN SHOW THE BACK OF THE BLUE PIECES
		LDA #$64			;LOAD $64 AND STORE IN ZPADDR+1, THAT WILL BE THE LOW BYTE OF THE VERA MEMORY ADDRESS TO LOAD (THE BACK SIDE OF THE BLUE GAME PIECES)
		STA ZPADDR+1
		JMP SHOWPIECE_1_NORMAL

SHOWPIECE_1_R_BACK
		LDA #$30			;LOAD $30 AND STORE IN ZPADDR+1, THAT WILL BE THE LOW BYTE OF THE VERA MEMORY ADDRESS TO LOAD (THE BACK SIDE OF THE RED GAME PIECES)
		STA ZPADDR+1
	
SHOWPIECE_1_NORMAL
		;STORE THE VERA VRAM ADDRESS OF THE SPRITE TO THE Z_HL PAIR, USED BY THE SETSPRITE FUNCTION
		LDA ZPADDR+1		;TAKE THE VERA MEMORY ADDRESS LOW BYTE FROM ZPADDR+5 (FROM GETSPRITEDATA)
		STA Z_L				;STORE IT IN THE LOW BYTE OF THE Z_HL PAIR.
		LDA #$08			;LOAD HEX $08 IN A
		STA Z_H				;STORE IN Z_HL HIGH BYTE, VERA MEMORY ADDRESS HIGH BYTE (BITSHIFTED >>5, SEE SETSPRITE) 

		;GET THE X COORDINATE FROM THE SPRITE DATA ARRAY (ZPADDR+2) AND MULTIPLY BY 16 AND ADD $30 (START LOCATION IS $30)
		LDA ZPADDR+2
		ASL 				;MULTIPLY BY 2
		ASL					;MULTIPLY BY 4
		ASL					;MULTIPLY BY 8
		ASL					;MULTIPLY BY 16
		CLC
		ADC #$20			;ADD $30 TO THE X COORDINATE
		STA Z_IXL			;STORE IN Z_IXL 
		
		;GET THE Y COORDINATE FROM THE SPRITE DATA ARRAY (ZPADDR+3) AND MULTIPLY BY 16 AND ADD $30 (START LOCATION IS $30)
		LDA ZPADDR+3
		ASL 				;MULTIPLY BY 2
		ASL					;MULTIPLY BY 4
		ASL					;MULTIPLY BY 8
		ASL					;MULTIPLY BY 16
		CLC
		ADC #$20			;ADD $30 TO THE X COORDINATE
		STA Z_IYL			;STORE IN Z_IYL 
		
		LDA ZPADDR 			;LOAD THE SPRITE NUMBER FROM ZPADDR
		JSR SetSprite		;SET THE SPRITE ON SCREEN
		
	PLA						;RETRIEVE A FROM THE ACCUMULATOR AGAIN
	
	RTS						;RETURN TO PREVIOUS CODE

;---------------------------------------------
;FUNCTION - GET SPRITE NUMBER BASED ON A XY POSITION, X IS X POSITION, Y IS Y POSITION
; SPRITE NUMBER IS RETURNED IN A
;---------------------------------------------
GETSPRITENUMBER
	TXA						;TRANSFER X POSITION TO ACCUMULATOR
	ASL						;MULTIPLY BY 2 (000XXXX0)
	ASL						;MULTIPLY BY 2 (00XXXX00)
	ASL						;MULTIPLY BY 2 (0XXXX000)
	ASL						;MULTIPLY BY 2 (XXXX0000)
	STA ZPADDR+16			;STORE TEMPORARY IN ZPADDR+16
	TYA						;TRANSFER THE Y POSITION TO A (0000YYYY)
	CLC
	ADC ZPADDR+16			;ADD THE X POSITION TO THE Y POSITION (XXXXYYYY)
	STA ZPADDR+16			;STORE IN ZPADDR+16
	
	;HERE WE NEED TO CHECK IF THE SELECTED SPOT ON THE PLAYFIELD IS USED BY ANOTHER PIECE
	STZ ZPADDR+1			;ZERO OUT ZPADDR+1, WE'LL USE THIS AS A LOOP COUNTER
	LDA #$04			
	STA ZPADDR+5			;STORE $04 IN ZPADDR+3, WE'LL USE THIS AS THE HIGH BYTE OF THE SPRITE DATA ARRAY
	LDA #1
	STA ZPADDR+4			;STORE 1 IN ZPADDR+2, WE'LL USE THIS FOR THE LOW BYTE OF THE SPRITE DATA ARRAY (OFFSET BY ONE SO WE GET THE SECOND BYTE OF THE ARRAY)
	
GETSPRITENUMBER_A
	LDA (ZPADDR+4)			;LOAD THE X AND Y POSITION OF THE PIECE IN THE SPRITE DATA ARRAY
	CMP ZPADDR+16		;COMPARE THIS XY POSITION TO THE XY POSITION WHERE WE WANT TO PLACE THE PIECE DOWN
	BEQ	GETSPRITENUMBER_B	;IF IT FINDS A MATCH, THEN THIS XY POSITION IS ALREADY TAKEN, CHECK NEXT BIT
	
	;IF THIS XY POSITION IS NOT TAKEN, WE NEED TO CHECK THE NEXT GAME PIECE
	LDX ZPADDR+1			;LOAD THE LOOP COUNTER
	CPX #79					;SEE IF WE HAVE ALREADY CHECKED ALL 80 GAME PIECES AND THEIR LOCATIONS
	BEQ GETSPRITENUMBERX	;IF WE HAVE CHECKED ALL, END THE LOOP, RETURNING NOTHING 
	;IF WE HAVENT CHECKED ALL 79 YET, THEN LOOP AGAIN
	INX
	STX ZPADDR+1			;INCREMENT THE PIECE/LOOP COUNTER
	LDA ZPADDR+4			;LOAD THE SPRITE ARRAY LOW BYTE
	CLC
	ADC #3
	STA ZPADDR+4			;ADD 3 TO IT AND STORE IT AGAIN, FOR THE NEXT LOOP
	JMP GETSPRITENUMBER_A	;AND LOOP AGAIN TO CHECK THE NEXT PIECE

GETSPRITENUMBER_B			;HERE WE FOUND A SPRITE MATCHING THE XY POSITION SUPPLIED
	;WHAT WE NEED TO DO NOW, IS TAKE THE LOW BYTE NOW STORED IN ZPADDR+4, SUBTRACT 1 (SO IT'S SET TO THE FIRST BYTE OF THE SPRITE ARRAY DATA)
	;AND THEN KEEP SUBTRACTING BY 3 (KEEPING TRACK HOW MANY TIMES WE DID THAT), ONCE WE HIT 0, WE KNOW HOW MANY SUBTRACTIONS IT TOOK
	;THE NUMBER OF SUBTRACT BY 3 TRIES WILL BE THE SPRITE NUMBER TO RETURN
	LDA ZPADDR+4
	SEC
	SBC #1					;LOAD THE LOW BYTE FROM ZPADDR+4 AND SUBTRACT 1.
	LDX #0					;EMPTY THE X REGISTER, THIS WILL BE OUR LOOP COUNTER	
GETSPRITENUMBER_C
	CMP #0					;CHECK IF THE LOW BYTE IS 0
	BEQ GETSPRITENUMBER_C_END	;WHEN YOU FIND IT, THE SPRITE NUMBER IS IN X
	;IF IT'S NOT 0 YET, THEN LETS SUBTRACT AND TRY AGAIN
	INX						;INCREASE X
	SEC
	SBC #3					;SUBTRACT 3 FROM THE LOW BYTE
	JMP GETSPRITENUMBER_C	;LOOP BACK, TRY AGAIN

GETSPRITENUMBER_C_END		;HERE WE FOUND THE SPRITE NUMBER (ITS IN X) , SO WE CAN RETURN
	TXA						;TRANSFER THE SPRITE NUMBER FROM X TO A TO RETURN
	RTS
	
GETSPRITENUMBERX
	LDA #255				;LOAD 255 IN A, WE'LL RETURN THIS TO PREVIOUS SUBROUTINE, TO LET IT KNOW NOTHING HAS BEEN FOUND
	RTS
	
;---------------------------------------------
;FUNCTION - GET SPRITE DATA, THIS FUNCTION GETS ALL SPRITE DATA BASED ON SPRITE NUMBER IN A. RESULTS ARE STORED IN ZPADDR - ZPADDR+6
;ZPADDR=SPRITE NR. ZPADDR+1=LOW BYTE OF VERA VRAM START ADDRESS (HIGH BYTE IS ALWAYS $08). ZPADDR+2=SPRITE X. ZPADDR+3=SPRITE Y. ZPADDR+4=SPRITE TYPE. ZPADDR+5=LOW BYTE OF THE FIRST BYTE OF SPRITE DATA ARRAY RAM ADDRESS. ZPADDR+6=DEAD. ZPADDR+7=FOUND. 
;---------------------------------------------
GETSPRITEDATA
	PHA					;STORE A IN THE STACK (BACKUP)
	PHX					;STORE X IN THE STACK (BACKUP)
	PHY					;STORE Y IN THE STACK (BACKUP)
	
		STA ZPADDR			;STORE SPRITE NUMBER IN ZPADDR
		
		;NOW WE NEED TO GET THE SPRITE TYPE, WHICH IS LOCATED AT (SPRITE NUMBER *3) + $0400 (START ADDRESS OF SPRITE DATA ARRAY) + 2 (FOR THE 3RD BYTE OF THE SPRITE DATA)
		;THEN IT'S LOCATED IN THE LOWER NIBBLE OF THE BYTES
		ASL					;SHIFT THE BITS TO THE LEFT (SPRITE NR * 2)
		CLC
		ADC ZPADDR			;ADD THE SPRITE NUMBER TO A (NOW IT'S SPRITE NUMBER * 3)
		STA ZPADDR+5		;STORE THE LOW BYTE OF THE SPRITE DATA ARRAY MEMORY ADDRESS IN ZPADDR+5
		CLC
		ADC #2				;ADD 2 TO THE ADRRESS (FOR THE 3RD BYTES OF THE ADDRESS)
		STA ZPADDR+6		;STORE THE ADDRESS IN ZPADDR+6 (TEMPORARILY, IT'LL BE THE LOW BYTE OF THE ADDRESS TO READ FROM RAM)
		LDA #SPRITEARRAYH	;LOAD $40 IN A (HIGH BYTE OF SPRITE DATA ARRAY)
		STA ZPADDR+7		;STORE $04 IN ZPADDR+7 (WHICH WILL BE THE HIGH BYTE WHEN WE READ FROM ZPADDR+5)
		LDA (ZPADDR+6)		;NOW LOAD THE 3RD BYTE OF THE SPRITE IN THE SPRITE DATA ARRAY ($04XX)
		AND #%00001111		;REMOVE THE TOP 4 BITS (WHICH CONTAIN DETECTED AND ALIVE FLAGS ,LEAVING ONLY THE SPRITE TYPE IN THE LOW NIBBLE)
		STA ZPADDR+4		;NOW ACTUALLY STORE THE SPRITE TYPE IN ZPADDR+4
		
		;NOW TO GET THE VERA VRAM START ADDRESS, WHICH ALWAYS STARTS AT $08XX (BITSHIFTED >>5, SEE SETSPRITE ROUTINE)
		;LOW BYTE RED TEAM = $00 TO $33 (BUT EVERY SPRITE IS 4 BYTES, SO THE HIGHEST SPRITE IS AT $30)
		;LOW BYTE BLUE TEAM = $34 TO $67 (BUT EVERY SPRITE IS 4 BYTES, THE THE HIGHEST SPRITE IS AT $64)
		;SO WE NEED TO TAKE THE (SPRITE TYPE * 4) AND ADD $34 (DEC 52) IF ITS THE BLUE TEAM (SPRITE NUMBER >= 40).
		LDA ZPADDR+4		;LOAD THE SPRITE TYPE IN A
		ASL					;BIT SHIFT TO THE LEFT (SPRITE TYPE * 2)	
		ASL					;BIT SHIFT TO THE LEFT (SPRITE TYPE * 4)
		STA ZPADDR+1		;TEMPORARILY STORE A TO ZPADDR+1
		
		LDA ZPADDR			;RELOAD THE SPRITE NUMBER IN A
		CMP #40				;COMPARE THE SPRITE NUMBER TO DEC 40
		BCC	GETSPRITEDATAB	;IF THE SPRITE NUMBER IS < 40 THEN WE DONT NEED TO ADD DEC 52 AS THE SPRITE IS THE RED TEAM SO BRANCH TO GETSPRITEDATAB 
		LDA ZPADDR+1		;IF THE SPRITE NUMBER IS >= 40 THEN WE NEED TO ADD $34 (DEC 52)
		CLC
		ADC #$34			;ADD $34 (DEC 52) SINCE THE SPRITE IS OF THE BLUE TEAM.
		STA ZPADDR+1		;AND STORE THE LOW BYTE OF THE SPRITES VERA VRAM ADDRESS IN ZPADDR+1
GETSPRITEDATAB
		LDA ZPADDR+1		;LOAD ZPADDR+1 AGAIN (IF WE JUMPED STRAIGHT FROM GETSPRITEDATAB, THEN WE NEED TO RELOAD ZPADDR+1)
		STA ZPADDR+1		;FINALLY WE STORE THE LOW BYTE OF THE SPRITES VERA VRAM ADDRESS IN ZPADDR+1 (AGAIN)
		
		;NOW WE NEED TO GET THE X AND Y LOCATION OF THE SPRITE. IT'S IN THE 2ND BYTE OF THE SPRITE'S DATA ARRAY. (XXXXYYYY)
		;THE SPRITE DATA ARRAY MEMORY IS STILL LOADED IN ZPADDR+6 AND ZPADDR+7. SO WE CHANGE ZPADDR+6 (LOW BYTE) BY 1 TO READ THE 2ND BYTE INSTEAD OF THE 3RD BYTES
		LDA ZPADDR+6		;LOAD THE LOW BYTE OF THE SPRITE DATA ARRAY'S MEMORY ADDRESS
		SEC
		SBC #1				;REDUCE VALUE BY 1
		STA ZPADDR+6		;STORE IN ZPADDR+5 AGAIN
		LDA (ZPADDR+6)		;LOAD THE 2ND BYTE OF THE SPRITE DATA ARRAY.
		STA ZPADDR+2		;TEMPORARILY STORE THE WHOLE BYTE IN ZPADDR+2 (XXXXYYYY)
		AND #%00001111		;REMOVE THE TOP 4 BITS (WHICH CONTAIN X COORDINATE, LEAVING ONLY THE Y COORDINATE IN THE LOW NIBBLE, 0000YYYY)
		STA ZPADDR+3		;STORE THE Y COORDINATE IN ZPADDR+3
		
		LDA ZPADDR+2		;RELOAD THE FULL BYTE (XXXXYYYY)
		;SWAP THE LOW AND HIGH NIBBLE AROUND
		ASL 				;(shift left - bottom bit zero)
		ADC #$80 			;(pop top bit off)
		ROL 				;(shift carry in)
		ASL 				;(shift left - bottom bit zero)
		ADC #$80			;(pop top bit off)
		ROL					;(shift carry in) 
		AND #%00001111		;REMOVE THE TOP 4 BITS (WHICH CONTAIN Y COORDINATE, LEAVING ONLY THE X COORDINATE IN THE LOW NIBBLE)
		STA ZPADDR+2
		
		;NOW WE NEED TO GET THE SPRITE FLAGS WHICH IS IN THE 3RD BYTE OF THE SPRITE DATA ARRAY BYTE (TOP NIBBLE) (DF--TTTT), D=DEAD, D=FOUND
		;THE ZPADDR+6 AND ZPADDR+7 STILL POINTS TO THE SPRITE DATA ARRAY 2ND BYTE ADDRESS. SO JUST ADD 1 AND RELOAD
		LDA ZPADDR+6		;LOAD THE LOW BYTE OF THE SPRITE DATA ARRAY.
		CLC
		ADC #1				;ADD 1 TO THE LOW BYTES
		STA ZPADDR+6		;STORE THE LOW BYTE IN ZPADDR+6
		LDA (ZPADDR+6)		;LOAD THE 3RD BYTE OF THE SPRITE DATA ARRAY
		STA ZPADDR+7		;TEMP STORE THE WHOLE BYTE IN ZPADDR+7
		
		AND #$80			;TEST FLAG BYTE 8 (DEAD)
		BNE DEADFLAGSET		;IF THE FLAG IS SET, BRANCH TO ALIVEFLAGSET
		LDA #0
		STA ZPADDR+6		;STORE 0 IN ZPADDR+6
		JMP ENDDEADFLAGSET	;JUMP TO FOUNDFLAG, OTHERWISE ZPADDR+6 WOULD BE SET TO 1 WHEN THAT SHOULD NOT HAPPEN...
DEADFLAGSET
		LDA #1
		STA ZPADDR+6		;STORE 1 IN ZPADDR+6
ENDDEADFLAGSET
		
		LDA ZPADDR+7		;RELOAD THE WHOLE BYTE FROM ZPADDR+7
		AND #$40			;TEST FLAG BYTE 7 (FOUND)
		BNE FOUNDFLAGSET	;IF THE FLAG IS SET, BRANCH TO DETECTEDFLAGSET
		LDA #0
		STA ZPADDR+7		;STORE 0 IN ZPADDR+7
		JMP ENDFOUNDFLAGSET	;JUMP TO FOUNDFLAG, OTHERWISE ZPADDR+6 WOULD BE SET TO 1 WHEN THAT SHOULD NOT HAPPEN...
FOUNDFLAGSET
		LDA #1
		STA ZPADDR+7		;STORE 1 IN ZPADDR+7	
ENDFOUNDFLAGSET
	
	;AND THAT'S IT, ALL VALUES READ AND STORED IN THE RIGHT ZPADDRESSES
	PLY					;RETREIVE Y FROM THE STACK
	PLX					;RETREIVE X FROM THE STACK
	PLA					;RETRIEVE A FROM THE STACK
GETSPRITEDATAY
	RTS					;RETURN TO PREVIOUS CODE

;---------------------------------------------
;FUNCTION - KILL SPRITE
;THIS FUNCTION WILL KILL THE SPRITE AND MOVE IT OFF THE SCREEN, SPRITE NUMBER IS PROVIDED IN A
;---------------------------------------------
KILLSPRITE:
	;WE ALSO NEED TO SET THE STATUS OF THE SPRITE/PIECE TO KILLED
	;FOR THAT WE NEED TO LOOP THROUGH THE SPRITE DATA ARRAY, THE 3RD BYTE OF EACH SPRITE UNTIL WE FIND THE SPRITE NUMBER, STORE SPRITE NUMBER IN ZPADDR+16 FOR NOW
	STA ZPADDR+16		;STORE THE SPRITE NUMBER IN ZPADDR+16 FOR NOW
	LDA #SPRITEARRAYH	;LOAD THE HIGH BYTE OF THE SPRITE DATA ARRAY
	STA ZPADDR+2		;STORE HIGH BYTE IN ZPADDR+2
	LDA #2				;LOAD 2 INTO THE LOW BYTE (THE 3RD BYTE OF THE SPRITE'S DATA)
	STA ZPADDR+1		;AND STORE THIS IN THE LOW BYTE (ZPADDR+1)
	LDX #0				;SET X AS 0 FOR LOOP COUNTER
	
KILLSPRITE_LOOP:		
	LDA (ZPADDR+1)		;LOAD THE SPRITE DATA FROM THE SPRITE DATA ARRAY
	CPX ZPADDR+16		;CHECK IF WE'RE ALREADY AT THE SPRITE DATA WE NEED TO GET
	BEQ KILLSPRITE_ENDLOOP	;IF WE FOUND IT, THEN END THE LOOPING
	INX					;IF WE DIDN'T THEN INCREMENT X
	LDA ZPADDR+1		
	CLC
	ADC	#3				;AND LOAD ZPADDR+1, INCREMENT BY THREE (FOR THE NEXT SPRITE DATA)
	STA ZPADDR+1		;AND STORE THE LOW BYTE AGAIN IN ZPADDR+1
	BRA KILLSPRITE_LOOP

KILLSPRITE_ENDLOOP:
	;NOW THAT WE HAVE FOUND THE SPRITE DATA (THE 3RD BYTE OF THE SPRITE'S DATA IS LOADED IN A NOW)
	;CHANGE BIT 7 (DEAD FLAG) TO BE DEAD :-)
	CLC
	ADC #128			;SET BIT 7 TO 1 (DECIMAL 128).
	STA (ZPADDR+1)		;AND STORE THE 3RD BYTE BACK TO MEMORY

	;NOW WE HAVE TO MOVE THE SPRITE OF THE SCREEN
	LDA ZPADDR+16		;LOAD THE SPRITE NUMBER
	LDX #10				;SET X POSITION TO 10
	LDY #10				;SET Y POSITION TO 10
	JSR MOVEPIECE		;MOVE THE SPRITE'S LOCATION OFF-SCREEN (XY=10)
	LDX #3
	STX VAR_SPRITESTATUS	;SET VAR_SPRITESTATUS TO 3 SO SHOWPIECE-SETPIECE SUB WILL DISABLE THE ACTUAL SPRITE
	JSR SHOWPIECE		;SHOW THE SPRITE IN IT'S NEW, OFFSCREEN LOCATION
	
	;AND THAT'S IT, GAME PIECE HAS BEEN FLAGGED AS KILLED AND MOVED OFF THE SCREEN.
	RTS
	
;---------------------------------------------
;FUNCTION - MOVE PIECE AT POS1 TO POS 2
;THIS FUNCTION WILL MOVE THE SPRITE AT CURSORACTION1 TO THE LOCATION IN CURSORACTION2
;---------------------------------------------
MOVEPOS1TOPOS2:
	LDA CURSORACTION1		;LOAD CURSORACTION1
	AND #%00001111			;APPLY BITMASK, LEAVING ONLY THE Y POSITION
	TAY						;TRANSFER THE Y POSITION TO THE Y REGISTER
	LDA CURSORACTION1		;LOAD CURSORACTION1 AGAIN
	AND #%11110000			;APPLY BITMASK, LEAVING ONLY THE X POSITION
	LSR						;BITSHIFT TO THE RIGHT (0XXXX000)
	LSR						;BITSHIFT TO THE RIGHT (00XXXX00)
	LSR						;BITSHIFT TO THE RIGHT (000XXXX0)
	LSR						;BITSHIFT TO THE RIGHT (0000XXXX)
	TAX						;TRANSFER THE X POSITION TO THE X REGISTER
	JSR GETSPRITENUMBER		;GET THE SPRITE NUMBER THAT IS AT CURSORACTION1'S LOCATION (SOURCE LOCATION)
	STA ZPADDR+10			;STORE THE SOURCE SPRITE NUMBER IN ZPDDR+10 FOR NOW
	
	LDA CURSORACTION2		;LOAD CURSORACTION2
	AND #%00001111			;APPLY BITMASK, LEAVING ONLY THE Y POSITION
	TAY						;TRANSFER THE Y POSITION TO THE Y REGISTER
	LDA CURSORACTION2		;LOAD CURSORACTION2 AGAIN
	AND #%11110000			;APPLY BITMASK, LEAVING ONLY THE X POSITION
	LSR						;BITSHIFT TO THE RIGHT (0XXXX000)
	LSR						;BITSHIFT TO THE RIGHT (00XXXX00)
	LSR						;BITSHIFT TO THE RIGHT (000XXXX0)
	LSR						;BITSHIFT TO THE RIGHT (0000XXXX)
	TAX						;TRANSFER THE X POSITION TO THE X REGISTER
	LDA ZPADDR+10			;LOAD A WITH THE SPRITE NUMBER WE FOUND EARLIER
	
	JSR MOVEPIECE			;MOVE THE GAME PIECE TO THE NEW LOCATION (VARIABLES ONLY)
	JSR SHOWPIECE			;AND SHOW THE PIECE AT IT'S NEW LOCATION
	RTS
	
;---------------------------------------------
;FUNCTION - MOVE CURSOR ON PLAYFIELD
;THIS FUNCTION WILL MOVE THE SELECT CURSOR OVER THE PLAYFIELD, UNTIL THE ENTER OR ESCAPE KEY IS PRESSED
;---------------------------------------------
MOVECURSOR
	;FIRST SHOW THE PLAYER CURSOR ON THE SCREEN
	LDA #80				;LOAD 80 IN A, WHICH IS THE SPRITE NUMBER FOR THE PLAYER CURSOR
	JSR SHOWPIECE		;SHOW THE GAME CURSOR ON THE SCREEN

MOVECURSORA
	JSR GETIN			;READ KEYBOARD USING A KERNAL CALL; BE AWARE! THIS WILL RESET X
	CMP #$0				;IF NO KEY HAS BEEN PRESSED.
	BEQ MOVECURSORA		;LOOP BACK TO CHECK KEYBOARD IF NO KEY HAS BEEN PRESSED

	;HIJACK THE INPUT TO DO SOME RND SEED REGENERATION, WHICH EVER KEY IS PRESSED
	PHA					;COPY THE KEY IN THE STACK FOR NOW
		CLC
		ADC SEED		;ADD THE OLD SEED TO THE KEY
		STA SEED		;STORE NEW SEED
	PLA 				;RELOAD THE KEY PRESSED IN A FROM STACK 
	
	CMP #$11			;IF THE CURSOR DOWN BUTTON IS PRESSED ($11 OR DEC 17)
	BNE MOVECURSORCHECKUP	;IF ITS NOT THE DOWN CURSOR, THEN CHECK IF ITS THE UP CURSOR
	;IF IT IS THE DOWN CURSOR, WE NEED TO CHECK IF WE ARE NOT ALREAD AT THE BOTTOM OF THE PLAYFIELD (Y = 9)
	LDA #80				;LOAD SPRITE NUMBER 80 IN ACCUMULATOR (THE PLAYER CURSOR)
	JSR GETSPRITEDATA	;LOAD THE DATA FOR THIS SPRITE
	LDA ZPADDR+3		;LOAD ZPADDR+3, WHICH CONTAINS THE SPRITE'S Y POSITION
	CMP #9				;CHECK IF IT'S ALREADY AT THE BUTTOM = 9
	BEQ MOVECURSORA	;IF IT IS, WE DON'T DO ANYTHING, LOOP BACK TO READ FOR ANOTHER KEYPRESS
	LDA #80				;LOAD SPRITE NUMBER 80 IN A
	LDX ZPADDR+2		;LOAD THE X POSITION OF THE SPRITE
	LDY ZPADDR+3		;LOAD THE Y POSITION OF THE SPRITE
	INY					;INCREMENT Y BY ONE, SINCE IT'S NOW ONE SPOT LOWER
	JSR MOVEPIECE		;MOVE THE SPRITE ONE DOWN
	JSR SHOWPIECE		;SHOW THE SPRITE AT IT'S NEW LOCATION
	JMP MOVECURSORA		;AND BACK TO CHECKING MORE KEYS
	
MOVECURSORCHECKUP
	CMP #$91			;IF THE CURSOR UP BUTTON IS PRESSED ($91 OR DEC 145)
	BNE MOVECURSORCHECKLFT	;IF ITS NOT THE UP CURSOR, THEN CHECK IF ITS THE LEFT CURSOR BUTTON
		;IF IT IS THE UP CURSOR, WE NEED TO CHECK IF WE ARE NOT ALREAD AT THE TOP OF THE PLAYFIELD (Y = 0)
	LDA #80				;LOAD SPRITE NUMBER 80 IN ACCUMULATOR (THE PLAYER CURSOR)
	JSR GETSPRITEDATA	;LOAD THE DATA FOR THIS SPRITE
	LDA ZPADDR+3		;LOAD ZPADDR+3, WHICH CONTAINS THE SPRITE'S Y POSITION
	CMP #0				;CHECK IF IT'S ALREADY AT THE TOP = 0
	BEQ MOVECURSORA	;IF IT IS, WE DON'T DO ANYTHING, LOOP BACK TO READ FOR ANOTHER KEYPRESS
	LDA #80				;LOAD SPRITE NUMBER 80 IN A
	LDX ZPADDR+2		;LOAD THE X POSITION OF THE SPRITE
	LDY ZPADDR+3		;LOAD THE Y POSITION OF THE SPRITE
	DEY					;DECREASE Y BY ONE, SINCE IT'S NOW ONE SPOT HIGHER
	JSR MOVEPIECE		;MOVE THE SPRITE UP DOWN
	JSR SHOWPIECE		;SHOW THE SPRITE AT IT'S NEW LOCATION
	JMP MOVECURSORA		;AND BACK TO CHECKING MORE KEYS
	
MOVECURSORCHECKLFT	
	CMP #$9D			;IF THE CURSOR LEFT BUTTON IS PRESSED ($9D OR DEC 157)
	BNE MOVECURSORCHECKRGT	;IF ITS NOT THE LEFT CURSOR, THEN CHECK IF ITS THE RIGHT CURSOR BUTTON
		;IF IT IS THE LEFT CURSOR, WE NEED TO CHECK IF WE ARE NOT ALREAD AT THE LEFT OF THE PLAYFIELD (X = 0)
	LDA #80				;LOAD SPRITE NUMBER 80 IN ACCUMULATOR (THE PLAYER CURSOR)
	JSR GETSPRITEDATA	;LOAD THE DATA FOR THIS SPRITE
	LDA ZPADDR+2		;LOAD ZPADDR+2, WHICH CONTAINS THE SPRITE'S X POSITION
	CMP #0				;CHECK IF IT'S ALREADY AT THE LEFT = 0
	BEQ MOVECURSORA	;IF IT IS, WE DON'T DO ANYTHING, LOOP BACK TO READ FOR ANOTHER KEYPRESS
	LDA #80				;LOAD SPRITE NUMBER 80 IN A
	LDX ZPADDR+2		;LOAD THE X POSITION OF THE SPRITE
	LDY ZPADDR+3		;LOAD THE Y POSITION OF THE SPRITE
	DEX					;DECREASE Y BY ONE, SINCE IT'S NOW ONE SPOT HIGHER
	JSR MOVEPIECE		;MOVE THE SPRITE UP DOWN
	JSR SHOWPIECE		;SHOW THE SPRITE AT IT'S NEW LOCATION
	JMP MOVECURSORA		;AND BACK TO CHECKING MORE KEYS
	
MOVECURSORCHECKRGT
	CMP #$1D			;IF THE CURSOR RIGHT BUTTON IS PRESSED ($1D OR DEC 29)
	BNE MOVECURSORCHECKOTH	;IF ITS NOT THE RIGHT CURSOR, THEN IT'S ANOTHER KEY
		;IF IT IS THE RIGHT CURSOR, WE NEED TO CHECK IF WE ARE NOT ALREAD AT THE RIGHT OF THE PLAYFIELD (X = 9)
	LDA #80				;LOAD SPRITE NUMBER 80 IN ACCUMULATOR (THE PLAYER CURSOR)
	JSR GETSPRITEDATA	;LOAD THE DATA FOR THIS SPRITE
	LDA ZPADDR+2		;LOAD ZPADDR+2, WHICH CONTAINS THE SPRITE'S X POSITION
	CMP #9				;CHECK IF IT'S ALREADY AT THE RIGHT = 9
	BEQ MOVECURSORA	;IF IT IS, WE DON'T DO ANYTHING, LOOP BACK TO READ FOR ANOTHER KEYPRESS
	LDA #80				;LOAD SPRITE NUMBER 80 IN A
	LDX ZPADDR+2		;LOAD THE X POSITION OF THE SPRITE
	LDY ZPADDR+3		;LOAD THE Y POSITION OF THE SPRITE
	INX					;DECREASE Y BY ONE, SINCE IT'S NOW ONE SPOT HIGHER
	JSR MOVEPIECE		;MOVE THE SPRITE UP DOWN
	JSR SHOWPIECE		;SHOW THE SPRITE AT IT'S NEW LOCATION
	JMP MOVECURSORA		;AND BACK TO CHECKING MORE KEYS

MOVECURSORCHECKOTH
	;LDA #80
	;JSR GETSPRITEDATA	;LOAD THE DATA FOR THIS SPRITE (CURSOR), SO WE HAVE ACCESS TO IT'S XY POSITION	
	JMP MOVECURSORY		;END MOVING CURSOR, RETURN BACK TO PREVIOUS FUNCTION	

MOVECURSORY
	RTS					;RETURN BACK TO PREVIOUS CODE

;---------------------------------------------
;FUNCTION - BLINK CURSOR (BLINKS A _ CHARACTER EVERY SECOND ON/OFF)
;---------------------------------------------
;NORMALLY SKIP OVER THIS FUNCTION
BLINKCUR
	JSR CLOCKAPI		;CALL THE COMMANDER X16 CLOCK API, THE RESULTS WILL BE PLACED IN ZP
	LDA	VAR_SEC			;LOAD THE AMOUNT OF SECONDS PREVIOUSLY STORED
	CMP CLOCKSEC		;COMPARE TO THE AMOUNT OF SECONDS CURRENTLY IN THE CLOCK (RESULT FROM THE CLOCKAPI CALL)
	BEQ BLINKCURY		;IF ITS THE SAME AMOUNT OF SECONDS, THEN NOTHING NEEDS TO CHANGE, JUMP TO END OF SUBROUTINE
						;OTHERWISE THE SECONDS ARE DIFFERENT, WE NEED TO CHANGE THE BLINKING CURSOR
	LDA VAR_BLINK		;LOAD THE CURRENT VALUE OF THE BLINKING CURSOR (0=OFF/1=ON)
	CMP #0
	BEQ BLINKCURA		;IF THE CURSOR IS CURRENTLY OFF, JUMP TO TURN IT ON
	LDA #0
	STA VAR_BLINK		;STORE THE VALUE 0 TO THE BLINKING CURSOR (OFF)
	BEQ BLINKCURB		;JUMP TO BLINKCURB, WE USE BEQ BECAUSE ZERO FLAG WILL BE SET IN LDA COMMAND
	
BLINKCURA
	LDA #1
	STA VAR_BLINK		;STORE THE VALUE 1 TO THE BLINKING CURSOR (ON)

BLINKCURB				;HERE WE NEED TO ACTUALLY PRINT A CURSOR (OR NOT) AT THE LOCATION OF THE CURSOR X
	LDA CLOCKSEC
	STA VAR_SEC			;STORE THE CURRENT AMOUNT OF SECONDS IN VAR_SEC, FOR THE NEXT TIME WE CHECK THE BLINKING CURSOR SUB
	
	LDA VAR_BLINK		;LOAD THE VALUE OF THE BLINKER IN A
	CMP #0				;IF IT'S A 0 THEN PRINT A SPACE AT THE LOCATION OF THE CURSOR
	BEQ BLINKCURC
	
	LDA #'.'
	JSR PrintChar		;OTHERWISE PRINT A _ IN THE LOCATION OF THE CURSOR
	JMP BLINKCURD
	
BLINKCURC
	LDA #' '
	JSR PrintChar		;OTHERWISE PRINT SPACE IN THE LOCATION OF THE CURSOR
	
BLINKCURD
	LDA CursorX	
	SEC					;ADD CARRY, OR THE SUBTRACT WILL NOT SUBTRACT 1 BUT 2 FROM CURSOR X
	SBC #1
	STA CursorX			;SUBTRACT 1 FROM THE CURSOR X LOCATION AND SAVE IT AGAIN. SO THE USER INPUT WIL OVERWRITE THE _ CHARCTER ON SCREEN.
	
BLINKCURY
	RTS					;RETURN TO PREVIOUS CODE

;---------------------------------------------
;FUNCTION - READ KEYBOARD BUFFER FOR 1 CHARACHTERS MAX)
;---------------------------------------------
CINPUT

CINPUTA
	JSR BLINKCUR		;BLINK THE CURSOR WHILE WE WAIT FOR USER INPUT
	JSR GETIN			;READ KEYBOARD USING A KERNAL CALL; BE AWARE! THIS WILL RESET X
	CMP #$0				;IF NO KEY HAS BEEN PRESSED.
	
	BEQ CINPUTA			;LOOP BACK TO CHECK KEYBOARD IF NO KEY HAS BEEN PRESSED
	LDX CHRBUFC			;RELOAD X FROM BACKUP IN CHRBUFC AS X WAS RESET BY JSR GETIN
	
	;HIJACK THE INPUT TO DO SOME RND SEED REGENERATION
	PHA					;COPY THE KEY IN THE STACK FOR NOW
		CLC
		ADC SEED		;ADD THE OLD SEED TO THE KEY
		STA SEED		;STORE NEW SEED
	PLA 				;RELOAD THE KEY PRESSED IN A FROM STACK 
	;RND REGENERATION DONE!
	
	;CMP #KENTER			;CHECK IF ENTER KEY IS ENTERED AS KEY, IF SO FINISH BY ADDING $FF AND LEAVE FUNCTION
	;BEQ LINPUTB
	
	STA CHRBUF			;IF ANYTHING ELSE THEN ENTER KEY. STORE KEY IN MEMORY BUFFER, OFFSET BY X VALUE
	RTS					;AND DONE, RETURN BACK TO
	
;---------------------------------------------
;FUNCTION - READ KEYBOARD BUFFER UNTIL ENTER KEY IS PRESSED (30 CHARACHTERS MAX)
;---------------------------------------------
;NORMALLY SKIP OVER THIS FUNCTION
LINPUT
	LDX #0				;RESET THE X COUNTER, KEEP TRACK OF NUMBER OF LETTERS ENTERED
	STX CHRBUFC
LINPUTA
	
	JSR BLINKCUR		;BLINK THE CURSOR WHILE WE WAIT FOR USER INPUT
	JSR GETIN			;READ KEYBOARD USING A KERNAL CALL; BE AWARE! THIS WILL RESET X
	CMP #$0				;IF NO KEY HAS BEEN PRESSED.
	
	BEQ LINPUTA			;LOOP BACK TO CHECK KEYBOARD IF NO KEY HAS BEEN PRESSED
	LDX CHRBUFC			;RELOAD X FROM BACKUP IN CHRBUFC AS X WAS RESET BY JSR GETIN
	
	;HIJACK THE INPUT TO DO SOME RND SEED REGENERATION
	PHA					;COPY THE KEY IN THE STACK FOR NOW
		CLC
		ADC SEED		;ADD THE OLD SEED TO THE KEY
		STA SEED		;STORE NEW SEED
	PLA 				;RELOAD THE KEY PRESSED IN A FROM STACK 
	;RND REGENERATION DONE!
	
	CMP #KENTER			;CHECK IF ENTER KEY IS ENTERED AS KEY, IF SO FINISH BY ADDING $FF AND LEAVE FUNCTION
	BEQ LINPUTB
	
	STA CHRBUF,X		;IF ANYTHING ELSE THEN ENTER KEY. STORE KEY IN MEMORY BUFFER, OFFSET BY X VALUE
	INX					;INCREMENT NUMBER OF STORED CHARACTERS X
	STX CHRBUFC			;STORE X (NUMBER OF CHARACTERS ENTERED) IN CHRBUFC, AS X WILL GET RESET THE NEXT TIME JSR GETIN IS CALLED.
	CPX #30				;CHECK IF THIS WAS CHARACTER NUMBER 30, THEN WE FINISH AS WELL (RETURN KEY)
	BEQ LINPUTB
	JSR PrintChar		;OTHERWISE PRINT KEY ON SCREEN AND RETURN TO LOOP TO CAPTURE ANOTHER KEY
	JMP LINPUTA			;ALWAYS JUMP
LINPUTB
	LDA #255			;FINISH OFF THE CHARACTER BUFFER BY ADDING $FF BEHIND THE LAST LETTER IN MEMORY
	STA CHRBUF,X
	STX CHRBUFC			;AND STORE NUMBER OF CHARACHTERS IN LAST BYTE
LINPUTY					;SUBROUTINE TERMINATOR (GO BACK WHERE YOU CAME FROM BEFORE!)
	RTS

;---------------------------------------------
;FUNCTION - COPY CHARACTER BUFFER TO VAR (X AND Y WILL CONTAIN DESTINATION MEMORY ADDRESS)
;---------------------------------------------
CHRBUFTOVAR
	STX ZPADDR			;TEMP STORE START MEM ADRES OF TEXT IN $30 AND $31
	STY ZPADDR+1
	LDY #0				;RESET Y FOR LOOPING THROUGH THE STRING
	;LDX #0				;RESET X FOR LOOPING THROUGH THE CHRBUFFER
CHRBUFTOVARA
	LDA CHRBUF,Y		;LOAD CHAR FROM BUFFER
	STA (ZPADDR),Y		;STORE CHAR IN VARIABLE
	CMP #255
	BEQ CHRBUFTOVARY	;COMPARE THE CHAR TO 255, IF IT IS, END SUBROUTINE
	;INX					
	INY					;IF NOT 255 THEN INCREMENT Y
	;CLV
	BNE CHRBUFTOVARA	;AND TRANSFER NEXT CHAR (USING BNE BECAUSE THE ZERO FLAG WILL NEVER BE SET IN INY)
CHRBUFTOVARY			;SUBROUTINE TERMINATOR (GO BACK WHERE YOU CAME FROM BEFORE!
	RTS

;---------------------------------------------
;FUNCTION - WIPE A LINE OF TEXT (FILL WITH SPACEBARS), LINE NUMBER SUPPLIED IN A
;---------------------------------------------
WIPELINE
	STA CursorY			;CHANGE CURSOR Y
	STZ CursorX			;RESET CURSOR X

	;NOW WIPE THE WHOLE LINE WITH BLANK SPACES
	LDA #" "
	LDX #0
WIPELINEA
	JSR PrintChar		;PRINT SPACEBAR
	INX
	CPX #40				;CHECK IF 40 CHARACTERS HAVE BEEN PRINTED ON THE LINE ALREADY
	BEQ WIPELINEY
	;CLV
	BNE WIPELINEA		;OTHERWISE PRINT AGAIN, WE BRANCH USING BNE BECAUSE ZERO FLAG WILL NOT BE SET IN LAST CPX COMMAND
WIPELINEY				;DONE WIPING THE LINE
	RTS

;---------------------------------------------
;FUNCTION - RANDOM NUMBER GENERATOR, SETS THE RND SEED TO A NUMBER OF 1 TO MAX IN ACCUMULATOR
;RETURNS THE RANDOM NUMBER IN THE ACCUMULATOR
;---------------------------------------------
;NORMALLY SKIP OVER THIS FUNCTION
RAND
	PHX				
	PHY						;STORE X AND Y TO STACK AS BACKUP
		CLC					;CLEAR THE CARRY BEFORE ADD COMMAND BELOW
		ADC #1				;ADD 1 TO THE VALUE ACCUMULATOR (WE NEEDED THE CLC FOR THIS)
		STA SEED+1			;TEMP STORE MAX NUMBER WE WANT TO GENERATE IN SEED+1
		
RANDA
		JSR RERND 			;GENERATE A RANDOM NUMBER, RESULT IS IN THE ACCUMULATOR
		CMP SEED+1			;COMPARE THE RANDOM NUMBER TOO THE NUMBER WE WANTED (TO BE BELOW THE MAX MUMBER) IN SEED+1
		BCC RANDY			;IF IT'S AT OR BELOW THE NUMBER WE WANTED NOW, DONE!
		JMP RANDA			;IF IT'S MORE THEN THE NUMBER WE WANTED, FIND ANOTHER NUMBER
RANDY
	PLY
	PLX						;RESTORE X AND Y FROM BACKUP
	RTS

;---------------------------------------------
;FUNCTION - NUMBER RE-RANDOMIZER
;---------------------------------------------
RERND
	;SIMPLE 8-BIT SEED RE-RANDOMIZER (ACTUALLY A LINEAR FEEDBACK SHIFT REGISTER)
	;(FROM HERE: https://codebase64.org/doku.php?id=base:small_fast_8-bit_prng )
	LDA SEED			;LOAD THE SEED INTO ACCUMULATOR
    BEQ doEor
	ASL					;SHIFT ALL BITS TO THE LEFT, MULTIPLY BY 2
	BEQ NOEOR			
    BCC NOEOR
doEor:
	EOR #$1D 			;IF THE CARRY WAS SET, DO A EOR/XOR WITH VALUE $1D
NOEOR:	
	STA SEED			;RESTORE THE SEED, NUMBER IS NOW REGENERATED
	
	RTS

;---------------------------------------------
;FUNCTION - RANDOM NUMBER GENERATOR INITIALIZER
;---------------------------------------------
;NORMALLY SKIP OVER THIS FUNCTION
RNDINIT
	;WE'RE GOING TO INITIALLY SEED THE RND USING THE CLOCK (SECONDS)
	JSR CLOCKAPI		;RUN THE COMMANDER X16 CLOCK API CALL. 
	LDA CLOCKSEC 		;LOAD THE CURRENT CLOCK SECONDS IN A
	STA SEED			;STORE THE SECONDS AS THE SEED.
	
	CMP #0				;CHECK IF THE SEED CONTAINS 0, IF SO, WE'LL SEED THE SEED WITH USER INPUT
	BNE RNDINITY		;IF IT'S 0, THEN WE DONT NEED TO TAKE ADDITIONAL SEEDING STEPS. OTHERWISE SKIP
	LDA #<SEEDASK
	STA z_L
	LDA #>SEEDASK
	STA z_H
	JSR PrintString		;PRINT THE QUESTION TO ASK FOR THE DATE
	JSR LINPUT			;CAPTURE USER INPUT, IT'S ONLY USED TO RESEED THE SEED, NOT STORED.
	
RNDINITY
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
SetVeraHML:		;Select Memory address %----AAAA YYYYYYYYY XXXXXXXX
				;Set Inc to            %AAAA----
						
		sta V_H	
		sty V_M
		stx V_L
	rts


AddVHML_YX		;add 24 bit %00000000 YYYYYYYYY XXXXXXXX to V_HML
		lda #0
AddVHML_AYX		;add 24 bit %AAAAAAAA YYYYYYYYY XXXXXXXX to V_HML
		pha
			clc
			txa
			adc V_L
			sta V_L
			tya
			adc V_M
			sta V_M
		pla
		adc V_H
		sta V_H
		rts	

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		
;THIS FUNCTION IS RESPONSIBLE FOR PRINTING CHARACTERS ON THE SCREEN	
PrintChar:
	SEC						
	SBC #32					;SUBTRACT 32 FROM THE CHARACTER VALUE TO PRINT. NOTE; OUR CHARACHTER HAS NO SPACE
	STA Z_AS				;STORE IN Z_AS (BACKUP REGISTER)
	
	PHX						
	PHY						;BACKUP THE REGSITERS WE USE, BOTH X AND Y
	LDA Z_H
	PHA
	LDA Z_L					
	PHA						;LOAD THE Z_HL PAIR AND STORE BOTH IN THE STACK AS BACKUPS.
		LDA Z_AS			;RELOAD THE CHARACTER VALUE TO PRINT (SUBTRACED BY 32); 8 bytes per character in our font
		STZ Z_H				;SET Z_H TO 0				
		ASL					;SHIFT THE BITS CHARACTER VALUE TO THE LEFT (MULTIPLY * 2)
		ROL	Z_H				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_H (AND SHIFT ALL THE BITS IN Z_H TO THE LEFT)
		ASL					;SHIFT THE BITS IN THE ACCUMULATOR TO THE LEFT AGAIN (MULTIPLY * 4) 
		ROL	Z_H				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_H (AND SHIFT ALL THE BITS IN Z_H TO THE LEFT)
		ASL 				;SHIFT THE BITS IN THE ACCUMULATOR TO THE LEFT AGAIN (MULTIPLY * 8) 
		ROL	Z_H				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_H (AND SHIFT ALL THE BITS IN Z_H TO THE LEFT)
		
		ADC #<BitmapFont	;ADD THE START RAM ADDRESS (LOW BYTE) OF THE BITMAPFONT TO THE ACCUMULATOR
		STA Z_L				;STORE THE ACCUMULATOR VALUE IN Z_L (FOR USE AS THE LOW BYTE)
		LDA Z_H				;NOW LOAD THE Z_H VALUE IN THE ACCUMULATOR
		ADC #>BitmapFont	;ADD THE START RAM ADDRESS (HIGH BYTE) OF THE BITMAPFONT TO THE ACCUMULATOR
		STA Z_H				;STORE THE ACCUMULATOR IN Z_H	(FOR USE AS THE HIGH BYTE)
		
		;AT THIS POINT Z_HL CONTAINS THE START ADDRESS OF THE BITMAPFONT, OFFSET BY THE CHARACTER THAT HAS TO BE LOADED.

		;NOW WE NEED TO SET THE VERA VRAM (SCREEN) ADDRESS TO PUT THE PIXELS IN. (24-BITS ADDRESS)
		;FOR THE 24 BITS WE'LL USE THE Z_D, Z_E AND A REGISTERS
		STZ Z_D				;FIRST EMPTY THE Z_D REGISTER. THIS WILL BE THE HIGH BYTE OF VERA VRAM
			
		LDA CursorY	 		;LOAD THE CURRENT CURSOR Y LOCATION TO A. CursorY* 160*8 = Cursor Y * %00000000 00000101 00000000
		STA Z_E				;STORE THE Y POSITION IN Z_E, WHICH WILL BE THE THE MIDDLE BYTE OF VERA VRAM
		
		ASL					;BITSHIFT A (CURSOR Y) TO THE LEFT (MULTIPLY * 2)
		ROL Z_D				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_D (AND SHIFT ALL THE BITS IN Z_D TO THE LEFT)
		ASL					;BITSHIFT A (CURSOR Y) TO THE LEFT (MULTIPLY * 4)
		ROL Z_D				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_D (AND SHIFT ALL THE BITS IN Z_D TO THE LEFT)
		
		CLC
		ADC Z_E				;ADD THE VALUE IN Z_E (CURSOR Y POSITION) TO THE ACCUMULATOR  
		
		TAY					;TRANSFER THE ACCUMULATOR VALUE TO Y
		
		LDA Z_D				;LOAD Z_D (THE CARRY BITS THAT HAVE FALLEN OFF BEFORE) IN THE ACCUMULATOR
		ADC #0				;ADD 0 TO THE ACCUMULAR;  Add carry to H byte
		STA Z_D				;STORE THE CARRY FROM THE BITSHIFTS TO Z_D (HIGH BYTE OF VERA VRAM ADDRESS)
		
		STZ Z_E				;EMPTY THE Z_E REGISTER
		LDA CursorX			;LOAD THE CURSOR X POSITION IN A; CursorX* 4 = Cursor Y * %00000000 00000000 00000100
		
		ASL					;BITSHIFT A (CURSOR X) TO THE LEFT (MULTIPLY * 2)
		ROL Z_E				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_E (AND SHIFT ALL THE BITS IN Z_E TO THE LEFT)
		ASL					;BITSHIFT A (CURSOR X) TO THE LEFT (MULTIPLY * 4)
		ROL Z_E				;STORE THE BITS THAT HAVE FALLEN OF THE ACCUMULTOR IN THE SHIFT IN Z_E (AND SHIFT ALL THE BITS IN Z_E TO THE LEFT)
		
		TAX					;TRANSFER A TO X ; X contains LOW byte 
		
		tya 				;Add carry to M byte
		adc z_e				
		tay					;Y contains M byte
		
		lda z_d				;Add carry to H byte & set AutoInc
		adc #$10			;A contains H byte  
		
		jsr SetVeraHML		;Set Screen Address for the location where the character has to be printed on screen.
		
		
		ldy #0						;Line in Font Char
CharLineAgain:
		phy
			lda (z_hl),y
			sta z_as
			ldx #4				;4 Bytes (2xpixels) Per Line; IN THE COMMANDS BELOW, TWO PIXELS WILL BE LOADED INTO A AT A TIME!
CharPixelAgain:				
			LDA #0				;EMPTY THE ACCUMULATOR
			ROL Z_AS			;Pop a bit off our font byte (12345678 -> 23456780 AND CARRY CONTAINS 1)
			ROL					;BITSHIFT A LEFT, INSERT THE BIT POPPED OF THE Z_AS ABOVE (00000000 -> 00000001)
			ROL					;BITSHIFT A LEFT AGAIN (00000001 -> 00000010)
			ROL					;BITSHIFT A LEFT AGAIN (00000010 -> 00000100)
			ROL					;BITSHIFT A LEFT AGAIN (00000100 -> 00001000)
			ROL Z_AS			;Pop a bit off our font byte (23456780 -> 34567800 AND CARRY CONTAINS 2)
			ROL					;BITSHIFT A LEFT AND INSERT THE BIT POPPED OF Z_AS ABOVE (00001000 -> 00010001)
	
			ASL					;BITSHIFT ALL THE BITS ONE TO THE LEFT, SO WE DON'T PRINT IN PALLET COLOR #1 BUT PALLET COLOR #2 (00010001 -> 00100010)
			
			STA V_D1			;SEND BITS (TWO PIXELS) TO SCREEN RAM
			
			dex 
			bne CharPixelAgain	;Next Pixel
			
			ldy #>(160-4)
			ldx #<(160-4)
			jsr AddVHML_YX			;Move down screen line (160 byte)
		ply
		iny							;Move down a line in our font
		tya
		cmp #8
		bne CharLineAgain			;8 lines per character
		
		inc CursorX
		lda CursorX
		cmp #40						;40 Characters per screen line
		bne NoNewLine
		jsr NewLine
NoNewLine:	
	pla								;Restore the registers
	sta z_l
	pla
	sta z_h	
	ply
	plx
	rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;FUNCTION TO COPY RAM MEMORY TO VERA VRAM.

VLDIR:					;Copy z_BC bytes from z_HL to V_D1 
						;(address must be already selected)
        lda (z_HL)
        sta V_D1		;Copy byte to V_D1
		INC z_L
		BNE	VLDIR_SkipInc1	;JUMP TO VLDIR_SkipInc1, AS LONG AS Z_L IS NOT 0
		INC	z_H
VLDIR_SkipInc1:
		DEC z_C			;Repeat until z_BC =0
		BNE VLDIR
		LDA z_B
		BEQ	VLDIR_Done
		DEC z_B
		jmp VLDIR
VLDIR_Done:
		rts
		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;Use HardSprite A ... position to z_IX,z_IY... 
;				Show data at VRAM z_HL >>5
	
;USED TO ACTUALLY PRINT/SHOW SPRITES ON THE SCREEN, 
SetSprite:	
	
	;MULTIPLY SPRITE NUMBER BY 8 (BECAUSE EACH SPRITE IN VERA HAS 8 BYTES WORTH OF ATTRIBUTE DATA), AND ADD $50
	sta z_as		;STORE SPRITE NUMBER IN Z_AS
	lda #0			;EMPTY A
	asl z_as		;x2 - SHIFT THE BITS IN THE SPRITE NUMBER TO THE LEFT
	rol				;SHIFT THE BIT IN A TO THE LEFT, STORE ANY DROPPED BIT ON THE RIGHT BIT OF A
	asl z_as		;x4 - SHIFT THE BITS IN THE SPRITE NUMBER TO THE LEFT AGAIN
	rol				;SHIFT THE BIT IN A TO THE LEFT, STORE ANY DROPPED BIT ON THE RIGHT BIT OF A
	asl z_as		;x8 - SHIFT THE BIT IN THE SPRITE NUMBER TO THE LEFT FOR A 3RD TIME
	rol				;SHIFT THE BIT IN A TO THE LEFT, STORE ANY DROPPED BIT ON THE RIGHT BIT OF A
	
	clc				;CLEAR THE CARRY FLAG
	adc #$50		;Sprite settings base $F5000 - ADD HEX 50 (DEC 80) TO THE VALUE IN A
	tay				;TRANSFER A TO Y, FOR THE MIDDLE BYTE OF THE VERA ADDRESS
	ldx z_as		;TRANSFER Z_AS TO X, FOR THE LOW BYTE OF THE VERA ADDRESS
	lda #$1F		;$F5000 with Autoinc ($1F5000), LOAD $1F IN A, FOR THE TOP BYTE OF THE VERA ADDRESS 
	
	jsr SetVeraHML	;Set VERA to point to Sprite Nr Settings = $1F5000 for Sprite 0
	
	;NOW LETS SET THE ATTRIBUTE DATA OF THE SPRITE
	lda z_l			;128/4>>5 = %100 per sprite
	sta v_d1		;LLLLLLLL - Low BYTE address bits - $68
	
	lda z_h			; Addr: $10000>>5 = $800 ; Addr: $1A000>>5 = $D00
	sta v_d1		;C---HHHH  C=colordepth 
								;H=High address bits - $08
	lda z_ixl
	sta v_d1		;XXXXXXXX - X Lbits
		
	lda z_ixh
	sta v_d1		;------XX - X Hbits
	
	lda z_iyl
	sta v_d1		;YYYYYYYY - Y Lbits
	
	lda z_iyh
	sta v_d1		;------YY -  Y Hbits
	
	LDX VAR_SPRITESTATUS			;LOAD VAR_SPRITESTATUS. IT SHOULD CONTAIN THE SPRITE SIZE, 0=16X16,1=32X32,2=64X64,3=SPRITE DISABLED
	CPX #3
	BNE ENABLESPRITE
	;HERE WE'RE GOING TO DISABLE THE SPRITE, THEN SET THE VAR_SPRITESTATUS BACK 0 SO IT WILL PRINT A 16X16 SPRITE
		; CCCCZZVH Z=Zdepth C=collision V=Vdepth H=Hdepth
	lda #%00000000	;Z=0 (MEANING DISABLE SPRITE)
	sta v_d1	
	STZ VAR_SPRITESTATUS	;RESET VAR_SPRITESTATUS BACK TO 0, SO IT WILL PRINT A 16X16 SPRITE, 
	BRA CHECKSPRITE16X16	;JUMP TO CHECKSPRITE16X16, TO CHECK FOR THE SIZE OF THE SPRITE TO BE PRINTED
	
ENABLESPRITE:
		; CCCCZZVH Z=Zdepth C=collision V=Vdepth H=Hdepth
	lda #%00001100	;Z=3 (MEANING IN FRONT OF LAYER 0 AND 1, ALWAYS ON TOP)
	sta v_d1	
	
CHECKSPRITE16X16:
	CPX #0
	BNE CHECKSPRITE32X32	;CHECK IF THE SPRITE IS 0, IF NOT THEN HOP TO CHECKSPRITE32X32
		; HHWWPPPP - H=Height W=Width P=Palette 
	LDA #%01010000			;SET SPRITE SIZE 16X16 (HW 0123 = 8/16/32/64 px)
	JMP DONECHECKSPRITESIZE	;JUMP TO DONECHECKSPRITESIZE
CHECKSPRITE32X32:

	CPX #1
	BNE CHECKSPRITE64X64	;CHECK IF THE SPRITE IS 1, IF NOT THEN HOP TO CHECKSPRITE64X64
	    ; HHWWPPPP - H=Height W=Width P=Palette 
	LDA #%10100000			;SET SPRITE SIZE 32X32 HW 0123 = 8/16/32/64 px)
	JMP DONECHECKSPRITESIZE	;JUMP TO DONECHECKSPRITESIZE
	
CHECKSPRITE64X64:
		; HHWWPPPP - H=Height W=Width P=Palette 
	LDA #%11110000			;SET SPRITE SIZE 64X64 (HW 0123 = 8/16/32/64 px)

DONECHECKSPRITESIZE:
	STA v_d1		;SEND SPRITE HIGHT, WIDTH AND PALLETE TO V_DL
	
	rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

NewLine:		
	stz CursorX				;Zero X
	inc CursorY				;Increase Y
	rts