;---------------------------------------------
;FUNCTION - SHOW LEGEND, THIS WILL SHOW THE NAMES AND RANK OF THE PIECES AT THE LEGEND TO THE RIGHT OF THE SPRITES
;---------------------------------------------
SHOWLEGEND
	;NOW PRINT THE NEXT TO THE PLACEABLES SPRITES
	LDA #<LEGEND					;SHOW TEXT ON SCREEN
	STA z_L
	LDA #>LEGEND
	STA z_H
	JSR PrintString
	RTS								;RETURN TO PREVIOUS CODE
	
;---------------------------------------------
;FUNCTION - PHASE 2 CURSOR MOVE WRAPPER
; THIS FUNCTION WILL CATCH ANY KEY PRESSED (OTHER THEN THE CURSOR KEYS) FOR PHASE 2
; AND DECIDE TO RETURN TO PROCESS THE KEY, OR TO RETURN BACK TO THE MOVECURSOR SUBROUTINE
;---------------------------------------------
PHASE2MOVECURSORWRAPPER
	JSR MOVECURSOR		;FIRST MOVE THE CURSOR AROUND UNTIL A KEY OTHER THEN THE CURSOR KEYS IS PRESSED
	
	;THE LAST KEY PRESSED IS IN THE ACCUMULATOR
	CMP #$0D			;CHECK IF THE RETURN KEY HAS BEEN PRESSED
	BNE PHASE2MOVECURSORWRAPPER_0	;IF NOT, JUMP TO THE NEXT ONE
	JMP PHASE2MOVECURSORWRAPPERY	;IF IT IS, KEY ACCEPTED, END FUNCTION

PHASE2MOVECURSORWRAPPER_0
	;NONE OF THE KEYS PRESSED ARE ALLOWED, SO GO BACK TO MOVING THE CURSOR
	JMP PHASE2MOVECURSORWRAPPER
	
PHASE2MOVECURSORWRAPPERY
	STA CHRBUF						;STORE THE CURRENT KEY IN THE CHARACTER BUFFER
	RTS
	
;----------------------------------------------
;FUNCTION - CHECK VALID PIECE SELECTION
; THIS FUNCTION CHECKS IF THE 1ST SELECTED GAME PIECE IS ACTUALLY ABLE TO MOVE
;----------------------------------------------

;THERE ARE 9 CHECKS THAT HAVE TO BE PASSED TO CONSIDER THE MOVE VALID: HERE WE'RE GOING TO DO THE FIRST 4
	;1) THE SOURCE PIECE CAN'T BE A EMPTY FIELD (CAN'T MOVE A EMPTY FIELD AROUND)
	;2) THE SOURCE PIECE CAN'T BE ONE OF THE COMPUTER'S PIECES
	;3) THE SOURCE PIECE CAN'T BE A IMMOVABLE PIECE (BOMB/FLAG)
	;4) THE SOURCE PIECE CAN'T BE BOXED IN ON ALL SIDES (NO WHERE TO MOVE IT)

CHECKVALIDPIECE1
	LDA #80
	JSR GETSPRITEDATA		;LOAD THE SPRITE DATA FROM SPRITE 80 (THE PLAYER CURSOR)

	LDA ZPADDR+2			;LOAD THE X POSITION
	ASL						;MULTIPLY BY 2 (000XXXX0)
	ASL						;MULTIPLY BY 2 (00XXXX00)
	ASL						;MULTIPLY BY 2 (0XXXX000)
	ASL						;MULTIPLY BY 2 (XXXX0000)
	STA ZPADDR+10			;STORE TEMPORARY IN ZPADDR+10
	LDA ZPADDR+3			;LOAD THE Y POSITION (0000YYYY)
	CLC
	ADC ZPADDR+10			;ADD THE X POSITION TO THE Y POSITION (XXXXYYYY)
	STA CURSORACTION1		;STORE IN CURSORACTION1
	
;*** CHECK 1) THE SOURCE PIECE CAN'T BE A EMPTY FIELD (CAN'T MOVE A EMPTY FIELD AROUND)	
	;HERE WE NEED TO CHECK IF THE SELECTED SPOT ON THE PLAYFIELD IS USED BY ANOTHER PIECE
	STZ ZPADDR+1			;ZERO OUT ZPADDR+1, WE'LL USE THIS AS A LOOP COUNTER
	LDA #$04			
	STA ZPADDR+5			;STORE $04 IN ZPADDR+3, WE'LL USE THIS AS THE HIGH BYTE OF THE SPRITE DATA ARRAY
	LDA #1
	STA ZPADDR+4			;STORE 1 IN ZPADDR+2, WE'LL USE THIS FOR THE LOW BYTE OF THE SPRITE DATA ARRAY (OFFSET BY ONE SO WE GET THE SECOND BYTE OF THE ARRAY)
	
CHECKVALIDPIECE1_1
	LDA (ZPADDR+4)			;LOAD THE X AND Y POSITION OF THE PIECE IN THE SPRITE DATA ARRAY
	CMP CURSORACTION1		;COMPARE THIS XY POSITION TO THE XY POSITION WHERE WE WANT TO PLACE THE PIECE DOWN
	BEQ	CHECKVALIDPIECE1_1_FOUND	;IF IT FINDS A MATCH, THEN THIS XY POSITION IS ALREADY TAKEN, CHECK NEXT BIT
	
	;IF THIS XY POSITION IS NOT TAKEN, WE NEED TO CHECK THE NEXT GAME PIECE
	LDX ZPADDR+1			;LOAD THE LOOP COUNTER
	CPX #79					;SEE IF WE HAVE ALREADY CHECKED ALL 80 GAME PIECES AND THEIR LOCATIONS
	BEQ CHECKVALIDPIECE1_1_NOTFOUND	;IF WE HAVE, END THE LOOP, AND GO TO CHECKVALIDPIECE_1_NOTFOUND, 
	;IF WE HAVENT CHECKED ALL 79 YET, THEN LOOP AGAIN
	INX
	STX ZPADDR+1			;INCREMENT THE PIECE/LOOP COUNTER
	LDA ZPADDR+4			;LOAD THE SPRITE ARRAY LOW BYTE
	CLC
	ADC #3
	STA ZPADDR+4			;ADD 3 TO IT AND STORE IT AGAIN, FOR THE NEXT LOOP
	JMP CHECKVALIDPIECE1_1	;AND LOOP AGAIN TO CHECK THE NEXT PIECE

CHECKVALIDPIECE1_1_NOTFOUND	;IF WE DON'T FIND A MATCHING PIECE AT THE POSITION OF THE CURSOR, THEN THERE'S NOTHING TO MOVE
	LDA #<MOVEERR1			;SHOW ERROR ON SCREEN
	STA z_L
	LDA #>MOVEERR1
	STA z_H
	JSR PrintString
	JSR CINPUT				;WAIT FOR PLAYER TO HIT RETURN (WE WONT USE IT)
	LDA #255
	STA CURSORACTION1		;STORE 255 IN CURSORACTION1, SO THE PREVIOUS SUBROUTINE KNOWS THE CHECK FAILS
	RTS						;EXIT SUBROUTINE
	
CHECKVALIDPIECE1_1_FOUND 	;HERE WE FOUND A MATCHING PIECE AT THE SPOT THE CURSOR IS AT

;*** CHECK 2) THE SOURCE PIECE CAN'T BE ONE OF THE COMPUTER'S PIECES
	;NOW TO NEED TO CHECK IF THE PIECE FOUND IS ONE OF THE PLAYER'S PIECES
	LDX ZPADDR+2
	LDY ZPADDR+3			;LOAD X AND Y POSITION IN X AND Y REGISTERS
	JSR GETSPRITENUMBER		;GET SPRITE NUMBER, BASED ON THE X AND Y POSITION, NUMBER WILL BE RETURNED IN A
	STA ZPADDR+11			;STORE THE SPRITE NUMBER IN ZPADDR+11 FOR NOW
	
	;NOW THE SPRITE NUMBER SHOULD BE IN A
	CMP #40					;CHECK IF THE SPRITE NUMBER IS 40
	BCS	CHECKVALIDPIECE1_1_B	;IF THE SPRITE NUMBER IS EQUAL OR MORE THEN 40, THEN IT'S A BLUE TEAM PIECE
	;OTHERWISE ITS A RED TEAM PIECE
	LDA VAR_TEAM			;LOAD THE PLAYER'S TEAM
	CMP #1					;CHECK IF THE PLAYER'S TEAM IS BLUE
	BNE CHECKVALIDPIECE1_2	;IF IT DOESNT MATCH, (IF THE PLAYER IS IN THE RED TEAM AND MOVING A RED PIECE, GO ON TO THE NEXT CHECK 
	BEQ CHECKVALIDPIECE1_1_FAIL	;IF IT MATCHES (IF THE PLAYER IS IN THE BLUE TEAM AND TRYING TO MOVE A RED PIECE), SHOW ERROR

CHECKVALIDPIECE1_1_B		;IF IT'S A BLUE TEAM PIECE WE'RE MOVING
	LDA VAR_TEAM			;LOAD THE PLAYER'S TEAM
	CMP #0					;CHECK IF THE PLAYER'S TEAM IS RED
	BNE CHECKVALIDPIECE1_2	;IF IT DOESNT MATCH, (IF THE PLAYER IS IN THE BLUE TEAM AND MOVING A BLUE PIECE, GO ON TO THE NEXT CHECK 
	BEQ CHECKVALIDPIECE1_1_FAIL	;IF IT MATCHES (IF THE PLAYER IS IN THE RED TEAM AND TRYING TO MOVE A BLUE PIECE), SHOW ERROR

CHECKVALIDPIECE1_1_FAIL
	LDA #<MOVEERR2			;SHOW ERROR ON SCREEN
	STA z_L
	LDA #>MOVEERR2
	STA z_H
	JSR PrintString
	JSR CINPUT				;WAIT FOR PLAYER TO HIT RETURN (WE WONT USE IT)
	LDA #255
	STA CURSORACTION1		;STORE 255 IN CURSORACTION1, SO THE PREVIOUS SUBROUTINE KNOWS THE CHECK FAILS
	RTS						;EXIT SUBROUTINE

CHECKVALIDPIECE1_2

;*** CHECK 3) THE SOURCE PIECE CAN'T BE A IMMOVABLE PIECE (BOMB/FLAG)
	;AND NOW THE FINAL CHECK OF THE FIRST SELECTED PIECE, CHECKING IF YOU'RE TRYING TO MOVE A PIECE WHICH CAN'T BE MOVED (FLAG AND BOMB)
	LDA ZPADDR+11			;RELOAD THE TEMP STORED SPRITE NUMBER
	JSR GETSPRITEDATA		;GET THE SPRITE DATA FOR THE SELECTED SPRITE
	
	LDA ZPADDR+4			;LOAD THE SPRITE TYPE, RETURNED FROM THE GETSPRITEDATA SUBROUTINE
	CMP #0					;CHECK IF THE SPRITE TYPE IS THE BOMBS
	BEQ CHECKVALIDPIECE1_2_FAIL	;IF IT IS A BOMB, THEN SHOW ERROR
	CMP #1					;CHECK IF THE SPRITE TYPE IS THE FLAG
	BEQ CHECKVALIDPIECE1_2_FAIL ;IF IT IS A FLAG, THEN SHOW ERROR
	BRA CHECKVALIDPIECE1_3	;IF IT'S NOT A BOMB OR FLAG, THEN PROCEED TO THE NEXT CHECK
	
CHECKVALIDPIECE1_2_FAIL	
	LDA #<MOVEERR3			;SHOW ERROR ON SCREEN
	STA z_L
	LDA #>MOVEERR3
	STA z_H
	JSR PrintString
	JSR CINPUT				;WAIT FOR PLAYER TO HIT RETURN (WE WONT USE IT)
	LDA #255
	STA CURSORACTION1		;STORE 255 IN CURSORACTION1, SO THE PREVIOUS SUBROUTINE KNOWS THE CHECK FAILS
	RTS						;EXIT SUBROUTINE

CHECKVALIDPIECE1_3
;*** CHECK 4) THE SOURCE PIECE CAN'T BE BOXED IN ON ALL SIDES (NO WHERE TO MOVE IT)

	;NOW WE NEED TO CHECK ALL THE SPOTS AROUND THE SPRITE TO SEE IF ANY ARE FREE. 
	;X+1, X-1, Y+1, Y-1. IT DOESNT MATTER WHERE THE EMPTY SPOT IS, THAT'S FOR ANOTHER CHECK. 
	LDA ZPADDR+2			;LOAD CURSOR X POSITION
	SEC
	SBC #1					;SUBTRACT 1 FROM X
	TAX						;TRANSFER A TO X, TEMPORARY STORAGE OF A
	ASL
	ASL
	ASL
	ASL						;BITSHIFT X 4 BITS TO THE LEFT (XXXX0000)
	CLC
	ADC ZPADDR+3			;ADD THE Y POSITION TO THE ACCUMULATOR (XXXXYYYY)
	STA ZPADDR+10			;STORE IN ZPADDR+10 (X-1 POSITION STORED)
	
	TXA						;TRANSFER BACKUP BACK TO A
	CLC
	ADC #2					;ADD 2 TO X
	ASL
	ASL
	ASL
	ASL						;BITSHIFT X 4 BITS TO THE LEFT (XXXX0000)
	CLC
	ADC ZPADDR+3			;ADD THE Y POSITION TO THE ACCUMULATOR (XXXXYYYY)
	STA ZPADDR+11			;STORE IN ZPADDR+11 (X+1 POSITION STORED)
	
	
	LDA ZPADDR+2			;LOAD THE X POSITION OF THE CURSOR
	ASL
	ASL
	ASL
	ASL						;BITSHIFT THE X POSITION 4 BITS TO THE LEFT (XXXX0000)
	STA ZPADDR+13			;TEMP STORE IN ZPADDR+13
	
	LDA ZPADDR+3			;LOAD Y POSITION OF THE CURSOR
	SEC
	SBC #1					;SUBTRACT 1 FROM Y
	CLC
	ADC ZPADDR+13			;ADD THE X POSITION FOR IT (XXXXYYYY)
	STA ZPADDR+12			;STORE IN ZPADDR+12 (Y-1)
	
	LDA ZPADDR+3			;LOAD THE Y POSITION OF THE CURSOR AGAIN
	CLC
	ADC #1					;ADD 1 TO Y
	CLC
	ADC ZPADDR+13			;ADD THE X POSITION TO IT (XXXXYYYY)
	STA ZPADDR+13			;STORE IN ZPADDR+13 (X+1)
	
	;NOW THAT WE HAVE THE FOUR POSITIONS IN ZPADDR+10 TO ZPADDR+13, WE NEED TO CHECK THEM
	LDY #10					;RESET Y TO 10 FOR USE AS A LOOPCOUNTER FOR THE FIRST LOOP
	STY ZPADDR+14			;STORE LOOP 1 COUNTER IN ZPADDR+14 FOR NOW
CHECKVALIDPIECE1_3_L1		;FIRST LOOP, FOR RUNNING THROUGH THE 4 LOCATIONS ON THE BOARD TO CHECK
	LDA ZPADDR,Y			;LOAD THE POSITION TO CHECK
	STA CURSORACTION2		;STORE THE POSITION TO CHECK IN CURSORACTION2 FOR NOW, SINCE IT'S NOT BEING USED AT THE MOMENT.
	
	LDA #$04
	STA ZPADDR+5			;LOAD #$04 IN THE HIGH BYTE AND STORE IN ZPADDR+5
	LDA #1
	STA ZPADDR+4			;LOAD #1 IN THE LOW BYTE AND STORE IN ZPADDR+4
	LDX #0					;ZERO OUT THE LOOP COUNTER FOR THE SPRITE DATA ARRAY
	STX ZPADDR+1			;STORE LOOP COUNTER IN ZPADDR+1 FOR NOW
CHECKVALIDPIECE1_3_L2		;SECOND LOOP, FOR RUNNING THROUGH THE XY POSITIONS IN THE SPRITE ARRAY TO SEE IF WE CAN FIND A MATCH
	LDA (ZPADDR+4)			;LOAD THE X AND Y POSITION OF THE PIECE IN THE SPRITE DATA ARRAY
	
	CMP CURSORACTION2		;COMPARE THIS XY POSITION TO THE XY POSITION WHERE WE COULD PLACE THE PIECE DOWN
	BEQ	CHECKVALIDPIECE1_3_L2_FOUND	;IF IT FINDS A MATCH, THEN THIS XY POSITION IS ALREADY TAKEN, CHECK NEXT BIT
	
	;IF THIS XY POSITION IS NOT TAKEN, WE NEED TO CHECK THE NEXT GAME PIECE
	LDX ZPADDR+1			;LOAD THE LOOP COUNTER
	CPX #79					;SEE IF WE HAVE ALREADY CHECKED ALL 80 GAME PIECES AND THEIR LOCATIONS
	BEQ CHECKVALIDPIECE1_3_L2_NOTFOUND	;IF WE HAVE, END THE LOOP, AND GO TO CHECKVALIDPIECE_1_NOTFOUND, 
	;IF WE HAVENT CHECKED ALL 79 YET, THEN LOOP AGAIN
	INX
	STX ZPADDR+1			;INCREMENT THE PIECE/LOOP COUNTER
	LDA ZPADDR+4			;LOAD THE SPRITE ARRAY LOW BYTE
	CLC
	ADC #3
	STA ZPADDR+4			;ADD 3 TO IT AND STORE IT AGAIN, FOR THE NEXT LOOP
	BRA CHECKVALIDPIECE1_3_L2	;AND LOOP AGAIN TO CHECK THE NEXT PIECE
	
CHECKVALIDPIECE1_3_L2_FOUND	;HERE WE FOUND A MATCH FOR THE XY POSITION, AKA THE POSITION IS TAKEN

	;NOW WE NEED TO CHECK IF THE LOCATION FOUND IS A PIECE OF THE COMPUTER'S TEAM (BECAUSE THAT'S A VALID MOVE!)
	;X STILL CONTAINS THE SPRITE NUMBER OF THE SPRITE CHECKED WHEN THE LOCATION WAS FOUND
	LDA VAR_TEAM			;LOAD THE TEAM
	CMP #0
	BEQ CHECKVALIDPIECE1_3_L2_FOUND_R	;IF IT MATCHES 0, THEN THE COMPUTERS PIECES ARE 40-79
	CPX #40								;IF NOT ,THEN THE COMPUTERS PIECES ARE 0-39. CHECK IF THE SPRITE NUMBER IS 40.
	BCC CHECKVALIDPIECE1_3_VALID		;IF IT'S LESS THEN 40, THEN IT'S A COMPUTER'S PIECE, THAT'S A VALID MOVE, END CHECK 4!
	BRA CHECKVALIDPIECE1_3_L2_FOUND_INVALID	;IF IT'S NOT, THEN ITS A PLAYER'S PIECE, INVALID LOCATION, SO LETS CHECK THE NEXT LOCATION
CHECKVALIDPIECE1_3_L2_FOUND_R
	CPX	#40
	BCS	CHECKVALIDPIECE1_3_VALID	;IF THE SPRITE CHECKED IS 40 OR HIGHER, THEN IT'S A VALID MOVE, THUS WE CAN END CHECK 4.
	;IF ITS 39 OR LESS, THEN IT'S A RED TEAM PIECE, THAT A INVALID SPOT, MOVE ON TO TRY THE NEXT LOCATION (OR END)
CHECKVALIDPIECE1_3_L2_FOUND_INVALID	

	;KNOWING THIS, WE CAN MOVE ON TO THECK THE OTHER LOCATIONS AROUND THE PIECE. IF THIS WAS ALREADY THE 4TH LOCATION CHECKED, 
	;THEN PRINT A ERROR OF A INVALID MOVE, UNMOVABLE PIECE.
	LDY ZPADDR+14			;LOAD THE LOOP 1 COUNTER
	CPY #13					;CHECK IF IT'S ALREADY CHECKED ALL FOUR LOCATIONS
	BEQ CHECKVALIDPIECE1_3_ERR	;IF IT HAS, THEN PRINT ERROR AND RETURN
	;IF IT HASN'T CHECKED ALL 4 LOCATIONS YET, THEN LOAD THE NEXT ONE AND RETURN TO LOOP AGAIN
	INY						;INCREMENT Y BY ONE
	STY ZPADDR+14			;STORE THE Y OFFSET IN ZPADDR+14 AGAIN
	BRA CHECKVALIDPIECE1_3_L1	;BRANCH BACK TO LOOP 1.

CHECKVALIDPIECE1_3_ERR
	;SO WE FOUND XY SPRITE MATCHES FOR ALL 4 LOCATIONS, SO THE PIECE CAN'T BE MOVED
	LDA #<MOVEERR4			;SHOW ERROR ON SCREEN
	STA z_L
	LDA #>MOVEERR4
	STA z_H
	JSR PrintString
	JSR CINPUT				;WAIT FOR PLAYER TO HIT RETURN (WE WONT USE IT)
	LDA #255
	STA CURSORACTION1		;STORE 255 IN CURSORACTION1, SO THE PREVIOUS SUBROUTINE KNOWS THE CHECK FAILS
	RTS						;EXIT SUBROUTINE	

CHECKVALIDPIECE1_3_L2_NOTFOUND	;HERE WE WENT THROUGH ALL 79 SPRITES AND HAVE NOT FOUND A MATCH

	;SO HERE WE THEN NEED TO CHECK IF THE SPOT TO CHECK IS NOT A ILLIGAL LOCATION (OUTSIDE THE PLAYFIELD OR IN THE 2 OBSTACLES)
	LDA CURSORACTION2		;LOAD THE POSITION TO CHECK
	AND #%00001111			;APPLY A BITMASK TO FILTER OUT ONLY THE Y POSITION	
	CMP #15					;COMPARE THE Y POSITION TO 15, WHICH ONLY HAPPENS IF IT OVERFLOWED (0 MINUS 1 = 255, CROPPED DOWN TO 4 BITS IS 15)
	BEQ CHECKVALIDPIECE1_3_L2_FOUND_INVALID	;IF Y IS 15, THEN THE LOCATION IS INVALID, CHECK THE NEXT LOCATION
	CMP #10					;COMPARE THE Y POSITION TO 11, WHICH IS BELOW THE PLAYFIELD
	BEQ CHECKVALIDPIECE1_3_L2_FOUND_INVALID	;IF Y IS 11, THEN THE LOCATION IS INVALID, CHECK THE NEXT LOCATION
	;NOW THAT Y HAS BEEN CHECKED, LETS DO THE SAME FOR X
	LDA CURSORACTION2		;LOAD THE POSITION TO CHECK
	AND #%11110000			;APPLY A BITMASK TO FILTER OUT ONLY THE X POSITION
	LSR						;BIT SHIFT TO THE RIGHT (0XXXX000)
	LSR						;BIT SHIFT TO THE RIGHT (00XXXX00)
	LSR						;BIT SHIFT TO THE RIGHT (000XXXX0)
	LSR						;BIT SHIFT TO THE RIGHT (0000XXXX)
	CMP #15					;COMPARE THE X POSITION TO 15, WHICH ONLY HAPPENS IF IT OVERFLOWED (0 MINUS 1 = 255, CROPPED DOWN TO 4 BITS IS 15)
	BEQ CHECKVALIDPIECE1_3_L2_FOUND_INVALID	;IF X IS 15, THEN THE LOCATION IS INVALID, CHECK THE NEXT LOCATION
	CMP #10					;COMPARE THE X POSITION TO 11, WHICH IS BELOW THE PLAYFIELD
	BEQ CHECKVALIDPIECE1_3_L2_FOUND_INVALID	;IF X IS 11, THEN THE LOCATION IS INVALID, CHECK THE NEXT LOCATION
	
	;NOW THE OUTSIDE OF THE PLAYFIELD HAS BEEN TESTED, NOW WE NEED TO CHECK THE TWO OBSTACLES IN THE MIDDLE OF THE FIELD.
	;WE HAVE A LOOKUP TABLE WITH THE ILLIGAL POSITIONS IN THERE (STARTING AT INVALIDPOS)
	LDY #0					;RESET Y AS LOOP COUNTER FOR THE LOOKUP TABLE
CHECKVALIDPIECE1_L3
	LDA INVALIDPOS,Y		;LOAD THE VALUE FROM THE INVALIDPOS LOOKUP TABLE
	
	CMP CURSORACTION2		;CHECK THE VALUE FROM THE INVALIDPOS LOOKUP TABLE TO THE XY POSITION IN CURSORACTION1
	BEQ CHECKVALIDPIECE1_3_L2_FOUND_INVALID	;IF A MATCH HAS BEEN FOUND, THEN THE POSITION IN CURSORACTION1 IS INVALID!
	
	CPY #7					;CHECK IF WE'VE ALREADY CHECKED ALL 7 POSITIONS
	BEQ CHECKVALIDPIECE1_L3_DONE	;IF WE DID, THEN END THE LOOP
	INY						;INCREMENT Y
	BRA CHECKVALIDPIECE1_L3	;JUMP BACK TO THE LOOP
	
CHECKVALIDPIECE1_L3_DONE	;HERE WE CHECKED ALL 8 ILLIGAL POSITIONS TO THE POSITION IN CURSORACTION1, AND HAVEN'T FOUND IT
	;THEN WE FOUND A VALID MOVE, SO WE CAN EXIT WITH THIS VALID POSITION!
	
CHECKVALIDPIECE1_3_VALID	;THIS IS WHERE WE END IF IF WE FIND A VALID LOCATION TO MOVE TO, CHECK PASSED!
	;SUCCESS!, JUST RETURN THE XY POSITION ALREAD IN CURSORACTION1 AND RETURN TO THE PREVIOUS SUBROUTINE
	RTS

;----------------------------------------------
;FUNCTION - SHOW SELCURSOR
; THIS FUNCTION DISPLAYS THE PREVIOUSLY SELECTED CURSOR OVER THE GAMEPIECE SELECTED. 
;----------------------------------------------
SHOWSELCURSOR
	LDA #80
	JSR GETSPRITEDATA		;GET THE SPRITE DATA FROM THE PLAYER CURSOR'S
	
	LDA #81					;LOAD 81 IN A (WHICH IS THE SPRITE NUMBER FOR THE PLAYER PREVOUSLY SELECTED CURSOR)
	LDX ZPADDR+2			;LOAD THE CURSORS X POSITION IN REGISTER X
	LDY ZPADDR+3			;LOAD THE CURSORS Y POSITION IN REGISTER Y
	JSR MOVEPIECE			;MOVE THE SELECTED CURSOR (SPRITE 81) TO THE LOCATION OF THE PLAYER CURSOR
	JSR SHOWPIECE			;AND SHOW THE PIECE ON THE SCREEN
	RTS						;ALL DONE, RETURN TO PREVIOUS CODE!
	
;----------------------------------------------
;FUNCTION - CHECK VALID PIECE SELECTION, SELECTION 2
; THIS FUNCTION CHECKS IF THE 2ND SELECTED SPOT IS A VALID TO SPOT TO MOVE TO
;----------------------------------------------	

;THERE ARE 10 CHECKS THAT HAVE TO BE PASSED TO CONSIDER THE MOVE VALID: 
; WE ALREADY DID 4 CHECKS IN CHECKVALIDPIECE1. HERE WE'RE GOING TO DO THE REMAINING 6
;5) THE DESTINATION PIECE CAN'T BE THE SAME AS THE SOURCE PIECE
;6) THE DESTINATION PIECE CAN'T BE ONE OF THE PLAYER'S OWN PIECES (OTHER OPTIONS ARE A EMPTY FIELD OR A COMPUTER PIECE, BOTH ARE FINE)
;7) THE DESTINATION CAN'T BE ON THE OBSTACLES (THE CASTLES) IN THE MIDDLE OF THE FIELD
;8) THE DESTINATION CAN'T BE DIAGONAL FROM THE SOURCE LOCATION, ONLY X OR Y MOVEMENT ALLOWED, NOT BOTH.
;9) THE DESTINATION CAN'T BE MORE THEN 1 FIELD AWAY FROM THE SOURCE, UNLESS THE SOURCE PIECE IS A SCOUT
;10) IF SOURCE PIECE IS A SCOUT, CHECK THAT THE FIELDS IN BETWEEN THE SOURCE AND DESTINATION ARE EMPTY (JUMPING OVER PIECES IS NOT ALLOWED)

CHECKVALIDPIECE2
	LDA #80
	JSR GETSPRITEDATA		;LOAD THE SPRITE DATA FROM SPRITE 80 (THE PLAYER CURSOR)

	LDA ZPADDR+2			;LOAD THE X POSITION
	ASL						;MULTIPLY BY 2 (000XXXX0)
	ASL						;MULTIPLY BY 2 (00XXXX00)
	ASL						;MULTIPLY BY 2 (0XXXX000)
	ASL						;MULTIPLY BY 2 (XXXX0000)
	STA ZPADDR+10			;STORE TEMPORARY IN ZPADDR+10
	LDA ZPADDR+3			;LOAD THE Y POSITION (0000YYYY)
	CLC
	ADC ZPADDR+10			;ADD THE X POSITION TO THE Y POSITION (XXXXYYYY)
	STA CURSORACTION2		;STORE IN CURSORACTION2

;*** CHECK 5) THE DESTINATION PIECE CAN'T BE THE SAME AS THE SOURCE PIECE
	CMP CURSORACTION1		;COMPARE CURSORACTION1 TO CURSORACTION2
	BEQ CHECKVALIDPIECE2_5_FAIL	;IF IT MATCHES, THEN YOUR TRYING TO MOVE TO THE SAME LOCATION, CHECK FAILED!
	BRA CHECKVALIDPIECE2_6	;IF NOT, THEN CHECK PASSED, ON TO CHECK 6!

CHECKVALIDPIECE2_5_FAIL
	LDA #<MOVEERR5			;SHOW ERROR ON SCREEN
	STA z_L
	LDA #>MOVEERR5
	STA z_H
	JSR PrintString
	JSR CINPUT				;WAIT FOR PLAYER TO HIT RETURN (WE WONT USE IT)
	LDA #255
	STA CURSORACTION2		;STORE 255 IN CURSORACTION2, SO THE PREVIOUS SUBROUTINE KNOWS THE CHECK FAILS
	RTS						;EXIT SUBROUTINE
	
CHECKVALIDPIECE2_6
;*** CHECK 6) THE DESTINATION PIECE CAN'T BE ONE OF THE PLAYER'S OWN PIECES (OTHER OPTIONS ARE A EMPTY FIELD OR A COMPUTER PIECE, BOTH ARE FINE)
	LDA #$04
	STA ZPADDR+5			;LOAD #$04 IN THE HIGH BYTE AND STORE IN ZPADDR+5
	LDA #1
	STA ZPADDR+4			;LOAD #1 IN THE LOW BYTE AND STORE IN ZPADDR+4
	LDX #0					;ZERO OUT THE LOOP COUNTER FOR THE SPRITE DATA ARRAY
CHECKVALIDPIECE2_6_L
	LDA (ZPADDR+4)			;LOAD THE X AND Y POSITION OF THE PIECE IN THE SPRITE DATA ARRAY
	
	CMP CURSORACTION2		;COMPARE THIS XY POSITION TO THE XY POSITION WHERE WE COULD PLACE THE PIECE DOWN
	BEQ	CHECKVALIDPIECE2_6_L_FOUND	;IF IT FINDS A MATCH, THEN THIS XY POSITION IS ALREADY TAKEN, CHECK NEXT BIT
	
	;IF THIS XY POSITION IS NOT TAKEN, WE NEED TO CHECK THE NEXT GAME PIECE
	CPX #79					;SEE IF WE HAVE ALREADY CHECKED ALL 80 GAME PIECES AND THEIR LOCATIONS
	BEQ CHECKVALIDPIECE2_7	;IF WE HAVE, END THE LOOP, AND GO TO CHECKVALIDPIECE1_7, 
	;IF WE HAVENT CHECKED ALL 79 YET, THEN LOOP AGAIN
	INX
	LDA ZPADDR+4			;LOAD THE SPRITE ARRAY LOW BYTE
	CLC
	ADC #3
	STA ZPADDR+4			;ADD 3 TO IT AND STORE IT AGAIN, FOR THE NEXT LOOP
	BRA CHECKVALIDPIECE2_6_L	;AND LOOP AGAIN TO CHECK THE NEXT PIECE
	
CHECKVALIDPIECE2_6_L_FOUND
	;IF WE DID FIND A MATCH IN THE SPRITE DATA ARRAY, WE JUST HAVE TO MAKE SURE THE SPRITE ISN'T ONE OF THE PLAYERS SPRITES
	LDA VAR_TEAM			;LOAD THE TEAM
	CMP #0
	BEQ CHECKVALIDPIECE2_6_L_FOUND_R	;IF IT MATCHES 0, THEN THE COMPUTERS PIECES ARE 40-79
	CPX #40								;IF NOT ,THEN THE COMPUTERS PIECES ARE 0-39. CHECK IF THE SPRITE NUMBER IS 40.
	BCC CHECKVALIDPIECE2_7				;IF IT'S LESS THEN 40, THEN IT'S A COMPUTER'S PIECE, THAT'S A VALID MOVE, END CHECK 6, ON TO CHECK 7!
	BRA CHECKVALIDPIECE2_6_L_FOUND_INVALID	;IF IT'S NOT, THEN ITS A PLAYER'S PIECE, INVALID LOCATION!
CHECKVALIDPIECE2_6_L_FOUND_R
	CPX	#40
	BCS	CHECKVALIDPIECE2_7	;IF THE SPRITE CHECKED IS 40 OR HIGHER, THEN IT'S A VALID MOVE, THUS WE CAN END CHECK 6 AND MOVE ON TO CHECK 7.
	;IF ITS 39 OR LESS, THEN IT'S A RED TEAM PIECE, THAT A INVALID SPOT, MOVE ON TO TRY THE NEXT LOCATION (OR END)
CHECKVALIDPIECE2_6_L_FOUND_INVALID
	LDA #<MOVEERR6			;SHOW ERROR ON SCREEN
	STA z_L
	LDA #>MOVEERR6
	STA z_H
	JSR PrintString
	JSR CINPUT				;WAIT FOR PLAYER TO HIT RETURN (WE WONT USE IT)
	LDA #255
	STA CURSORACTION2		;STORE 255 IN CURSORACTION2, SO THE PREVIOUS SUBROUTINE KNOWS THE CHECK FAILS
	RTS						;EXIT SUBROUTINE

CHECKVALIDPIECE2_7
;*** CHECK 7) THE DESTINATION CAN'T BE ON THE OBSTACLES (THE CASTLES) IN THE MIDDLE OF THE FIELD
	;ALL WE NEED TO DO HERE IS CHECK IF THE DESTINATION LOCATION (CURSORACTION2) IS IN THE INVALIDPOS LOOKUP TABLE
	LDX #0					;LOAD X WITH 0, IT'LL BE OUR LOOPCOUNTER/OFFSET
CHECKVALIDPIECE2_7_LOOP
	LDA INVALIDPOS,X		;LOAD ONE POSITION FROM THE INVALIDPOS LIST
	CMP CURSORACTION2		;COMPARE THE INVALID POSITION TO THE POSITION IN CURSORACTION2
	BEQ CHECKVALIDPIECE2_8_X_ERR	;IF IT'S A MATCH, THEN WE FOUND A INVALID POSITION. SHOW ERROR ON SCREEN
	;IF WE DIDN'T FIND A MATCH, WE CHECK IF X IS ALREADY 7,
	CPX #7					;CHECK IF LOOP COUNTER IS 7, WHICH MEANS WE CHECKED ALL 8 INVALID LOCATIONS
	BEQ CHECKVALIDPIECE2_8	;IF WE DID CHECK ALL 8 AND HAVE NOT FOUND A MATCH, CHECK 7 DONE, MOVE ON TO CHECK 8!
	INX						;IF WE HAVEN'T CHECKED ALL 8 POSITION, INCREMENT X
	BRA CHECKVALIDPIECE2_7_LOOP	;AND BRANCH BACK TO THE LOOP TO CHECK THE NEXT POSITION
	
CHECKVALIDPIECE2_8
	;IF THE DESTINATION COORDINATE HAS NO MATCH IN THE SPRITE DATA ARRAY, THEN CHECK 7 PASSED, ON TO CHECK 8!
;*** CHECK 8) THE DESTINATION CAN'T BE DIAGONAL FROM THE SOURCE LOCATION, ONLY X OR Y MOVEMENT ALLOWED, NOT BOTH.
	LDA #80
	JSR GETSPRITEDATA		;LOAD THE SPRITE DATA FROM THE CURSOR'S CURRENT POSITION
	LDA ZPADDR+2			;LOAD X POSITION OF CURSOR
	STA ZPADDR+10			;AND STORE CURSOR X IN ZPADDR+10
	LDA ZPADDR+3			;LOAD Y POSITION OF CURSOR
	STA ZPADDR+11			;STORE CURSOR Y POSITION IN ZPADDR+11
	
	LDA #81
	JSR GETSPRITEDATA		;LOAD THE SPRITE DATA FOR THE CURSOR'S PREVIOUS SELECTION
	LDA ZPADDR+2			;LOAD THE X POSITION OF THE PREVIOUS CURSOR
	SEC
	SBC ZPADDR+10			;SUBTRACT CURSOR X PREVIOUS POSITION FROM CURSOR X POSITION
	STA ZPADDR+12			;STORE THE X DIFFERENCE IN ZPADDR+12
	
	LDA ZPADDR+3			;LOAD Y POSITION FROM THE PREVIOUS CURSOR
	SEC
	SBC ZPADDR+11			;SUBTRACT CURSOR Y PREVIOUS POSITION FROM CURSOR Y POSITION
	STA ZPADDR+13			;STORE THE Y DIFFERENCE IN ZPADDR+13
	
	;NOW THAT WE HAVE CALCULATED THE DIFFERENCE IN X AND Y POSITION, WE NEED TO CHECK IF BOTH HAVE A VALUE OTHER THEN 0... IF SO, THEN THE PLAYER IS TRYING TO MOVE DIAGONALLY
	LDA ZPADDR+12			;LOAD THE X DIFFERENCE
	CMP #0					;CHECK IF THERE'S A X DIFFERENCE..
	BNE CHECKVALIDPIECE2_8_X_DIF	;IF THERE'S A DIFFERENCE, CHECK IF THERE'S A Y DIFFERENCE AS WELL...
	BEQ	CHECKVALIDPIECE2_9	;IF THERE'S NO X DIFFERENCE, THEN IT HAS TO HAVE ONLY A Y DIFFERENCE, END CHECK, AND GO TO CHECK 9
	
CHECKVALIDPIECE2_8_X_DIF	;IF THERE'S A X DIFFERENCE, WE NEED TO CHECK FOR A Y DIFFERENCE AS WELL
	LDA ZPADDR+13			;LOAD THE Y DIFFERENCE
	CMP #0					;CHECK IF THERE'S A Y DIFFERENCE..
	BEQ CHECKVALIDPIECE2_9	;IF THERE'S NO Y DIFFERENCE, THEN IT'S ALSO GOOD, GO TO CHECK 9!
CHECKVALIDPIECE2_8_X_ERR
	;HOWEVER, IF WE'RE HERE, THEN THERE'S A X AND Y DIFFERENCE!! SHOW ERROR
	LDA #<MOVEERR6			;SHOW ERROR ON SCREEN
	STA z_L
	LDA #>MOVEERR6
	STA z_H
	JSR PrintString
	JSR CINPUT				;WAIT FOR PLAYER TO HIT RETURN (WE WONT USE IT)
	LDA #255
	STA CURSORACTION2		;STORE 255 IN CURSORACTION2, SO THE PREVIOUS SUBROUTINE KNOWS THE CHECK FAILS
	RTS						;EXIT SUBROUTINE

CHECKVALIDPIECE2_9
;*** CHECK 9) THE DESTINATION CAN'T BE MORE THEN 1 FIELD AWAY FROM THE SOURCE, UNLESS THE SOURCE PIECE IS A SCOUT

	;FIND OUT OF CURSORACTION1'S POSITION CONTAINS A SPRITE WHICH IS A SCOUT
	;ALL WE NEED TO DO IS FIND THE SPRITE NUMBER THAT CORRESPONDS TO THE XY ADDRESS IN CURSORACTION1. AND WE HAVE THE SUB GETSPRITENUMBER FOR THAT
	LDA #81
	JSR GETSPRITEDATA		;LOAD THE SPRITE DATA FOR THE PREVIOUS ACTION CURSOR
	LDX ZPADDR+2			;LOAD THE X POSITION OF PREV-CURSOR IN THE X REGISTER
	LDY ZPADDR+3			;LOAD THE Y POSITION OF PREV-CURSOR IN THE Y REGISTER
	JSR GETSPRITENUMBER		;NOW GET THE CORRESPONDING SPRITE NUMBER THAT HAS THE SAME XY POSITION AS PREV-CURSOR
	;NOW THAT THE SPRITE NUMBER IS BEEN RETURNED IN A WE CAN JUST GET THE DATA FOR THAT SPRITE
	JSR GETSPRITEDATA		;GET SPRITE DATA FROM SPRITE THAT IS IN A...
	LDA ZPADDR+4			;LOAD THE SPRITE TYPE IN TO REGISTER A
	STA ZPADDR+14			;AND STORE THE SPRITE TYPE IN ZPADDR+14, WE'LL NEED IT A FEW TIMES LATER ON....
	
	;NOW LETS CHECK IF THE DISTANCE BETWEEN CURSORACTION1 AND CURSORACTION2 IS VALID
	LDA ZPADDR+12			;LOAD THE X DIFFERENCE (FROM CHECK 8)
	CMP #0					;CHECK IF THERE'S A X DIFFERENCE
	BEQ	CHECKVALIDPIECE2_9_Y	;IF THERE'S NOT, THEN CHECK THE Y DIFFERENCE
	CMP #1					;IF THERE IS A X DIFFERENCE, CHECK FOR A DIFFERENCE OF 1
	BEQ CHECKVALIDPIECE2_10	;IF THE X DIFFERENCE IS ONLY 1, THEN CHECK 9 PASSED, GO TO CHECK 10
	CMP #255				;NOW CHECK FOR 255, WHICH IS A DIFFERENCE OF -1 (ALSO ALLOWED)
	BEQ CHECKVALIDPIECE2_10	;IF THE X DIFFERENCE IS ONLY -1, THEN CHECK 9 PASSED, GO TO CHECK 10
	;IF WE GET HERE, THEN THERE'S A X DIFFERENCE OTHER THEN 1 OR 255 (-1), WHICH IS NOT ALLOWED UNLESS! 
	; THE SPRITE IN CURSORACTION1 IS A SCOUT!
	BRA CHECKVALIDPIECE2_9_SCOUTCHECK

CHECKVALIDPIECE2_9_Y	
	LDA ZPADDR+13			;LOAD THE Y DIFFERENCE
	CMP #1					;IF THERE IS A Y DIFFERENCE, CHECK FOR A DIFFERENCE OF 1
	BEQ CHECKVALIDPIECE2_10	;IF THE Y DIFFERENCE IS ONLY 1, THEN CHECK 9 PASSED, GO TO CHECK 10
	CMP #255				;NOW CHECK FOR 255, WHICH IS A DIFFERENCE OF -1 (ALSO ALLOWED)
	BEQ CHECKVALIDPIECE2_10	;IF THE Y DIFFERENCE IS ONLY -1, THEN CHECK 9 PASSED, GO TO CHECK 10
	;IF WE GET HERE, THEN THERE'S A X DIFFERENCE OTHER THEN 1 OR 255 (-1), WHICH IS NOT ALLOWED UNLESS! 
	; THE SPRITE IN CURSORACTION1 IS A SCOUT!
CHECKVALIDPIECE2_9_SCOUTCHECK
	LDA ZPADDR+14
	CMP #3					;CHECK IF THE VALUE IN ZPADDR+14 (THE SPRITE TYPE FROM CURSORACTION1) IS A SCOUT (TYPE=3)
	BNE	CHECKVALIDPIECE2_8_X_ERR	;IF IT'S NOT A SCOUT THEN JUST SHOW THE ERROR (FROM CHECK 8) AND EXIT SUBROUTINE		
	;IF IT DOES MATCH, THEN WE JUST GO ON TO CHECK 10
	
	
CHECKVALIDPIECE2_10
;*** CHECK 10) IF SOURCE PIECE IS A SCOUT, CHECK THAT THE FIELDS IN BETWEEN THE SOURCE AND DESTINATION ARE EMPTY (JUMPING OVER PIECES IS NOT ALLOWED)
	LDA ZPADDR+14
	CMP #3					;CHECK IF THE VALUE IN ZPADDR+14 (THE SPRITE TYPE FROM CURSORACTION1) IS A SCOUT (TYPE=3)
	BEQ CHECKVALIDPIECE2_10_S	;IF IT'S A SCOUT, HOP ON TO ACTUALLY CHECKING CHECK 10...
	;IF IT'S NOT A SCOUT, THEN WE PASSED CHECK 10, ALL CHECKS DONE! RETURN TO PREVIOUS SUB!
	RTS
	
CHECKVALIDPIECE2_10_S
	;NOW COMES THE TRICKY PART, WE NEED TO CHECK THE X DIFFERENCE FIRST, IF IT'S 1 (OR -1/255) THEN WE'RE DONE, ALL CHECKS PASSED. IF ITS 0 THEN WE DO THE SAME FOR Y POSITION.
	;HOWEVER IF X OR Y DIFF IS MORE THEN 0, WE NEED TO CHECK ALL THE FIELDS IN BETWEEN CURSORACTION1 AND CURSORACTION2, AND SEE IF THERE'RE EMPTY. 
	;IF ONE OF THE FIELDS HAS A PIECE IN IT, THEN THE CHECK FAILED AS THE SCOUT CAN JUMP MULTIPLE SQUARES, BUT NOT HOP OVER ANOTHER PIECE, ONLY EMPTY PIECES
	;LETS BEGIN BY CHECKING THE X DIFFERENCE FIRST
	LDA ZPADDR+12			;LOAD THE X DIFFERENCE (FROM CHECK 7)
	CMP #0					;CHECK IF THE DIFFERENCE IS 0
	BEQ CHECKVALIDPIECE2_10_S_Y	;IF IT IS 0, THEN THERE MUST BE A Y DIFFERENCE
	CMP #1					;OTHERWISE, CHECK IF THE X DIFFERENCE IS 1
	BEQ CHECKVALIDPIECE2_10_S_DONE	;IF IT IS, WE DONT NEED TO DO ANYMORE CHECKS, RETURN TO THE PREVIOUS SUB
	CMP #255				;OTHERWISE CHECK IF THE X DIFFERENCE IS -1 (/255)
	BEQ CHECKVALIDPIECE2_10_S_DONE	;IF IT IS, WE DONT NEED TO DO ANYMORE CHECKS, RETURN TO THE PREVIOUS SUB
	CMP #10					;COMPARE TO 10
	BCC CHECKVALIDPIECE2_10_S_X_1		;CHECK IF THE X DIFFERENCE LESS THEN 10 , THEN IT'S A POSITIVE MOVE (+1 TO +9), AS A NEGATIVE MOVE OF -9 WOULD BE 246... SO WE CAN'T CHECK TO HIGH
	;IF WE'RE HERE, THEN THE X DIFFERENCE IS LESS THEN -1...
	;A NEGATIVE X DIFFERENCE MEANS WE WANT TO MOVE TO THE RIGHT. CURSORACTION2 IS TO THE RIGHT OF CURSORACTION1
	;SO WE START A LOOP AT CURSORACTION1, AND MOVE TO THE RIGHT, STORING IT IN ZPADDR+14
	LDA CURSORACTION1
	STA ZPADDR+14
CHECKVALIDPIECE2_10_S_X_2_LOOP
	LDA ZPADDR+14			;LOAD ZPADDR+14 (TEMP XY LOCATION)
	CLC
	ADC #16					;ADD 16 TO THE XY LOCATION (SO THE 5TH BIT IS ADDED BY ONE, SO X IS OFFSET +1)
	CMP CURSORACTION2		;CHECK IF THE TEMP LOCATION IS THE SAME AS THE DESTINATION (CURSORACTION2)
	BEQ CHECKVALIDPIECE2_10_S_DONE	;IF IT IS, AND WE HAVEN'T FOUND A OCCUPIED SPOT IN THE MEAN TIME, THEN THE CHECK PASSED
	STA ZPADDR+14			;AND STORE IT AGAIN IN ZPADDR+14
	AND #%00001111			;APPLY BITMASK, LEAVING ONLY THE Y POSITION
	TAY						;TRANSFER THE REMAINING VALUE (Y POSITION) TO THE Y REGISTER
	LDA ZPADDR+14			;RELOAD THE XY POSITION
	AND #%11110000			;APPLY BITMASK, LEAVING ONLY THE X POSITION
	LSR						;SHIFT THE BITS TO THE RIGHT (0XXXX000)
	LSR						;SHIFT THE BITS TO THE RIGHT (00XXXX00)
	LSR						;SHIFT THE BITS TO THE RIGHT (000XXXX0)
	LSR						;SHIFT THE BITS TO THE RIGHT (0000XXXX)	
	TAX						;AND STORE THE REMAINING VALUE (X POSITION) TO THE X REGISTER
	JSR GETSPRITENUMBER		;FIND A SPRITE NUMBER BASED ON THESE XY POSITION (IN THE X AND Y REGISTER)
	CMP #255				;CHECK IF THE RETURNED SPRITE NUMBER IS 255, MEANING NONE FOUND AT THE X AND Y POSITION GIVEN
	BEQ CHECKVALIDPIECE2_10_S_X_2_LOOP	;IF IT IS 255, THEN A EMPTY SPOT IS FOUND, TRY AGAIN FOR THE NEXT SPOT
	JMP CHECKVALIDPIECE2_8_X_ERR	;IF IT'S NOT 255, THEN A SPRITE NUMBER IS RETURNED. SINCE THERE CAN'T BE A PIECE IN BETWEEN SOURCE AND DESTINATION, SHOW ERROR
	
CHECKVALIDPIECE2_10_S_DONE
	RTS						;CHECK 10 PASSED, ALL DONE! RETURN TO PREVIOUS SUB
	
CHECKVALIDPIECE2_10_S_X_1			;THEN WE NEED TO CHECK FIELDS WITH A POSITIVE X DIFFERENCE
	;A POSITIVE X DIFFERENCE MEANS WE WANT TO MOVE TO THE LEFT. CURSORACTION2 IS TO THE LEFT OF CURSORACTION1
	;SO WE START A LOOP AT CURSORACTION1, AND MOVE X TO THE LEFT, STORING IT IN ZPADDR+14
	LDA CURSORACTION1
	STA ZPADDR+14
CHECKVALIDPIECE2_10_S_X_1_LOOP
	LDA ZPADDR+14			;LOAD ZPADDR+14 (TEMP XY LOCATION)
	SEC
	SBC #16					;SUBTRACT 16 TO THE XY LOCATION (SO THE 5TH BIT IS SUBTRACTED BY ONE, SO X IS OFFSET -1)
	CMP CURSORACTION2		;CHECK IF THE TEMP LOCATION IS THE SAME AS THE DESTINATION (CURSORACTION2)
	BEQ CHECKVALIDPIECE2_10_S_DONE	;IF IT IS, AND WE HAVEN'T FOUND A OCCUPIED SPOT IN THE MEAN TIME, THEN THE CHECK PASSED
	STA ZPADDR+14			;AND STORE IT AGAIN IN ZPADDR+14
	AND #%00001111			;APPLY BITMASK, LEAVING ONLY THE Y POSITION
	TAY						;TRANSFER THE REMAINING VALUE (Y POSITION) TO THE Y REGISTER
	LDA ZPADDR+14			;RELOAD THE XY POSITION
	AND #%11110000			;APPLY BITMASK, LEAVING ONLY THE X POSITION
	LSR						;SHIFT THE BITS TO THE RIGHT (0XXXX000)
	LSR						;SHIFT THE BITS TO THE RIGHT (00XXXX00)
	LSR						;SHIFT THE BITS TO THE RIGHT (000XXXX0)
	LSR						;SHIFT THE BITS TO THE RIGHT (0000XXXX)	
	TAX						;AND STORE THE REMAINING VALUE (X POSITION) TO THE X REGISTER
	JSR GETSPRITENUMBER		;FIND A SPRITE NUMBER BASED ON THESE XY POSITION (IN THE X AND Y REGISTER)
	CMP #255				;CHECK IF THE RETURNED SPRITE NUMBER IS 255, MEANING NONE FOUND AT THE X AND Y POSITION GIVEN
	BEQ CHECKVALIDPIECE2_10_S_X_1_LOOP	;IF IT IS 255, THEN A EMPTY SPOT IS FOUND, TRY AGAIN FOR THE NEXT SPOT
	JMP CHECKVALIDPIECE2_8_X_ERR	;IF IT'S NOT 255, THEN A SPRITE NUMBER IS RETURNED. SINCE THERE CAN'T BE A PIECE IN BETWEEN SOURCE AND DESTINATION, SHOW ERROR
	
CHECKVALIDPIECE2_10_S_Y		;CHECK FOR WHICH Y DIFFERENCE THERE IS.
	LDA ZPADDR+13			;LOAD THE Y DIFFERENCE (FROM CHECK 7)
	CMP #1					;CHECK IF THE Y DIFFERENCE IS 1
	BEQ CHECKVALIDPIECE2_10_S_DONE	;IF IT IS, WE DONT NEED TO DO ANYMORE CHECKS, RETURN TO THE PREVIOUS SUB
	CMP #255				;OTHERWISE CHECK IF THE Y DIFFERENCE IS -1 (/255)
	BEQ CHECKVALIDPIECE2_10_S_DONE	;IF IT IS, WE DONT NEED TO DO ANYMORE CHECKS, RETURN TO THE PREVIOUS SUB
	CMP #10					;COMPARE TO 10
	BCC CHECKVALIDPIECE2_10_S_Y_1		;CHECK IF THE Y DIFFERENCE LESS THEN 10 , THEN IT'S A POSITIVE MOVE (+1 TO +9), AS A NEGATIVE MOVE OF -9 WOULD BE 246... SO WE CAN'T CHECK TO HIGH
	;IF WE GET HERE, IT'S A NEGATIVE Y DIFFERENCE OF LESS THEN -1!
	;A NEGATIVE Y DIFFERENCE MEANS WE WANT TO MOVE A PIECE DOWN. CURSORACTION2 IS LOWER THEN CURSORACTION1
	;SO WE START A LOOP AT CURSORACTION1, AND MOVE Y DOWN, STORING IT IN ZPADDR+14
	LDA CURSORACTION1
	STA ZPADDR+14
CHECKVALIDPIECE2_10_S_Y_2_LOOP
	LDA ZPADDR+14			;LOAD ZPADDR+14 (TEMP XY LOCATION)
	CLC
	ADC #1					;ADD 1 TO THE XY LOCATION (SO THE 1ST BIT IS ADDED BY ONE, SO Y IS OFFSET +1)
	CMP CURSORACTION2		;CHECK IF THE TEMP LOCATION IS THE SAME AS THE DESTINATION (CURSORACTION2)
	BEQ CHECKVALIDPIECE2_10_S_DONE	;IF IT IS, AND WE HAVEN'T FOUND A OCCUPIED SPOT IN THE MEAN TIME, THEN THE CHECK PASSED
	STA ZPADDR+14			;AND STORE IT AGAIN IN ZPADDR+14
	AND #%00001111			;APPLY BITMASK, LEAVING ONLY THE Y POSITION
	TAY						;TRANSFER THE REMAINING VALUE (Y POSITION) TO THE Y REGISTER
	LDA ZPADDR+14			;RELOAD THE XY POSITIN
	LSR						;SHIFT THE BITS TO THE RIGHT (0XXXXYYY)
	LSR						;SHIFT THE BITS TO THE RIGHT (00XXXXYY)
	LSR						;SHIFT THE BITS TO THE RIGHT (000XXXXY)
	LSR						;SHIFT THE BITS TO THE RIGHT (0000XXXX)	
	TAX						;AND STORE THE REMAINING VALUE (X POSITION) TO THE X REGISTER
	JSR GETSPRITENUMBER		;FIND A SPRITE NUMBER BASED ON THESE XY POSITION (IN THE X AND Y REGISTER)
	CMP #255				;CHECK IF THE RETURNED SPRITE NUMBER IS 255, MEANING NONE FOUND AT THE X AND Y POSITION GIVEN
	BEQ CHECKVALIDPIECE2_10_S_Y_2_LOOP	;IF IT IS 255, THEN A EMPTY SPOT IS FOUND, TRY AGAIN FOR THE NEXT SPOT
	JMP CHECKVALIDPIECE2_8_X_ERR	;IF IT'S NOT 255, THEN A SPRITE NUMBER IS RETURNED. SINCE THERE CAN'T BE A PIECE IN BETWEEN SOURCE AND DESTINATION, SHOW ERROR
	
CHECKVALIDPIECE2_10_S_Y_1
	;A POSITBLE Y DIFFERENCE MEANS WE WANT TO MOVE A PIECE UP. CURSORACTION2 IS HIGHER THEN CURSORACTION1
	;SO WE START A LOOP AT CURSORACTION1, AND MOVE Y UP, STORING IT IN ZPADDR+14
	LDA CURSORACTION1
	STA ZPADDR+14
CHECKVALIDPIECE2_10_S_Y_1_LOOP
	LDA ZPADDR+14			;LOAD ZPADDR+14 (TEMP XY LOCATION)
	SEC
	SBC #1					;SUBTRACT 1 FROM THE XY LOCATION (SO THE 1ST BIT IS SUBTRACTED BY ONE, SO Y IS OFFSET -1)
	CMP CURSORACTION2		;CHECK IF THE TEMP LOCATION IS THE SAME AS THE DESTINATION (CURSORACTION2)
	BEQ CHECKVALIDPIECE2_10_S_DONE	;IF IT IS, AND WE HAVEN'T FOUND A OCCUPIED SPOT IN THE MEAN TIME, THEN THE CHECK PASSED
	STA ZPADDR+14			;AND STORE IT AGAIN IN ZPADDR+14
	AND #%00001111			;APPLY BITMASK, LEAVING ONLY THE Y POSITION
	TAY						;TRANSFER THE REMAINING VALUE (Y POSITION) TO THE Y REGISTER
	LDA ZPADDR+14			;RELOAD THE XY POSITIN
	LSR						;SHIFT THE BITS TO THE RIGHT (0XXXXYYY)
	LSR						;SHIFT THE BITS TO THE RIGHT (00XXXXYY)
	LSR						;SHIFT THE BITS TO THE RIGHT (000XXXXY)
	LSR						;SHIFT THE BITS TO THE RIGHT (0000XXXX)	
	TAX						;AND STORE THE REMAINING VALUE (X POSITION) TO THE X REGISTER
	JSR GETSPRITENUMBER		;FIND A SPRITE NUMBER BASED ON THESE XY POSITION (IN THE X AND Y REGISTER)
	CMP #255				;CHECK IF THE RETURNED SPRITE NUMBER IS 255, MEANING NONE FOUND AT THE X AND Y POSITION GIVEN
	BEQ CHECKVALIDPIECE2_10_S_Y_1_LOOP	;IF IT IS 255, THEN A EMPTY SPOT IS FOUND, TRY AGAIN FOR THE NEXT SPOT
	JMP CHECKVALIDPIECE2_8_X_ERR	;IF IT'S NOT 255, THEN A SPRITE NUMBER IS RETURNED. SINCE THERE CAN'T BE A PIECE IN BETWEEN SOURCE AND DESTINATION, SHOW ERROR